@@ -2,7 +2,7 @@
   ==============================================================================
 
    This file is part of the JUCE examples.
-   Copyright (c) 2017 - ROLI Ltd.
+   Copyright (c) 2020 - Raw Material Software Limited
 
    The code included in this file is provided under the terms of the ISC license
    http://www.isc.org/downloads/software-support-policy/isc-license. Permission
@@ -116,7 +116,7 @@ public:
         if (thumbnail.getTotalLength() > 0)
         {
             auto newScale = jmax (0.001, thumbnail.getTotalLength() * (1.0 - jlimit (0.0, 0.99, amount)));
-            auto timeAtCentre = xToTime (getWidth() / 2.0f);
+            auto timeAtCentre = xToTime ((float) getWidth() / 2.0f);
 
             setRange ({ timeAtCentre - newScale * 0.5, timeAtCentre + newScale * 0.5 });
         }
@@ -229,12 +229,12 @@ private:
         if (visibleRange.getLength() <= 0)
             return 0;
 
-        return getWidth() * (float) ((time - visibleRange.getStart()) / visibleRange.getLength());
+        return (float) getWidth() * (float) ((time - visibleRange.getStart()) / visibleRange.getLength());
     }
 
     double xToTime (const float x) const
     {
-        return (x / getWidth()) * (visibleRange.getLength()) + visibleRange.getStart();
+        return (x / (float) getWidth()) * (visibleRange.getLength()) + visibleRange.getStart();
     }
 
     bool canMoveTransport() const noexcept
@@ -296,6 +296,7 @@ public:
 
         directoryList.setDirectory (File::getSpecialLocation (File::userHomeDirectory), true, true);
 
+        fileTreeComp.setTitle ("Files");
         fileTreeComp.setColour (FileTreeComponent::backgroundColourId, Colours::lightgrey.withAlpha (0.6f));
         fileTreeComp.addListener (this);
 
@@ -455,7 +456,7 @@ private:
        #endif
         {
             if (reader == nullptr)
-                reader = formatManager.createReaderFor (audioURL.createInputStream (false));
+                reader = formatManager.createReaderFor (audioURL.createInputStream (URL::InputStreamOptions (URL::ParameterHandling::inAddress)));
         }
 
         if (reader != nullptr)
@@ -497,14 +498,13 @@ private:
     {
         if (btn == &chooseFileButton && fileChooser.get() == nullptr)
         {
-            SafePointer<AudioPlaybackDemo> safeThis (this);
-
             if (! RuntimePermissions::isGranted (RuntimePermissions::readExternalStorage))
             {
+                SafePointer<AudioPlaybackDemo> safeThis (this);
                 RuntimePermissions::request (RuntimePermissions::readExternalStorage,
                                              [safeThis] (bool granted) mutable
                                              {
-                                                 if (granted)
+                                                 if (safeThis != nullptr && granted)
                                                      safeThis->buttonClicked (&safeThis->chooseFileButton);
                                              });
                 return;
@@ -515,24 +515,27 @@ private:
                 fileChooser.reset (new FileChooser ("Select an audio file...", File(), "*.wav;*.mp3;*.aif"));
 
                 fileChooser->launchAsync (FileBrowserComponent::openMode | FileBrowserComponent::canSelectFiles,
-                                          [safeThis] (const FileChooser& fc) mutable
+                                          [this] (const FileChooser& fc) mutable
                                           {
-                                              if (safeThis != nullptr && fc.getURLResults().size() > 0)
+                                              if (fc.getURLResults().size() > 0)
                                               {
                                                   auto u = fc.getURLResult();
 
-                                                  safeThis->showAudioResource (std::move (u));
+                                                  showAudioResource (std::move (u));
                                               }
 
-                                              safeThis->fileChooser = nullptr;
+                                              fileChooser = nullptr;
                                           }, nullptr);
             }
             else
             {
-                NativeMessageBox::showMessageBoxAsync (AlertWindow::WarningIcon, "Enable Code Signing",
-                                                       "You need to enable code-signing for your iOS project and enable \"iCloud Documents\" "
-                                                       "permissions to be able to open audio files on your iDevice. See: "
-                                                       "https://forum.juce.com/t/native-ios-android-file-choosers");
+                NativeMessageBox::showAsync (MessageBoxOptions()
+                                               .withIconType (MessageBoxIconType::WarningIcon)
+                                               .withTitle ("Enable Code Signing")
+                                               .withMessage ("You need to enable code-signing for your iOS project and enable \"iCloud Documents\" "
+                                                             "permissions to be able to open audio files on your iDevice. See: "
+                                                             "https://forum.juce.com/t/native-ios-android-file-choosers"),
+                                             nullptr);
             }
         }
     }
