@@ -2,7 +2,7 @@
   ==============================================================================
 
    This file is part of the JUCE examples.
-   Copyright (c) 2017 - ROLI Ltd.
+   Copyright (c) 2020 - Raw Material Software Limited
 
    The code included in this file is provided under the terms of the ISC license
    http://www.isc.org/downloads/software-support-policy/isc-license. Permission
@@ -102,7 +102,7 @@ private:
 
         burgerButton.setBounds (r.removeFromRight (40).withSizeKeepingCentre (20, 20));
 
-        titleLabel.setFont (Font (getHeight() * 0.5f, Font::plain));
+        titleLabel.setFont (Font ((float) getHeight() * 0.5f, Font::plain));
         titleLabel.setBounds (r);
     }
 
@@ -157,6 +157,10 @@ public:
         setApplicationCommandManagerToWatch (&commandManager);
         commandManager.registerAllCommandsForTarget (this);
 
+        // this ensures that commands invoked on the DemoRunner application are correctly
+        // forwarded to this demo
+        commandManager.setFirstCommandTarget (this);
+
         // this lets the command manager use keypresses that arrive in our window to send out commands
         addKeyListener (commandManager.getKeyMappings());
 
@@ -164,6 +168,8 @@ public:
         addAndMakeVisible (outerCommandTarget);
         addAndMakeVisible (sidePanel);
 
+        setWantsKeyboardFocus (true);
+
         setSize (500, 500);
     }
 
@@ -172,6 +178,8 @@ public:
        #if JUCE_MAC
         MenuBarModel::setMacMainMenu (nullptr);
        #endif
+
+        commandManager.setFirstCommandTarget (nullptr);
     }
 
     void resized() override
@@ -314,7 +322,11 @@ public:
     }
 
 private:
+   #if JUCE_DEMO_RUNNER
+    ApplicationCommandManager& commandManager = getGlobalCommandManager();
+   #else
     ApplicationCommandManager commandManager;
+   #endif
 
     std::unique_ptr<MenuBarComponent> menuBar;
     MenuBarPosition menuBarPosition = MenuBarPosition::window;
