@@ -571,7 +571,10 @@ struct MenuPage   : public Component
             struct CustomComponent  : public PopupMenu::CustomComponent
             {
                 CustomComponent (int widthIn, int heightIn, Colour backgroundIn)
-                    : idealWidth (widthIn), idealHeight (heightIn), background (backgroundIn)
+                    : PopupMenu::CustomComponent (false),
+                      idealWidth (widthIn),
+                      idealHeight (heightIn),
+                      background (backgroundIn)
                 {}
 
                 void getIdealSize (int& width, int& height) override
