@@ -605,22 +605,6 @@ private:
 namespace juce
 {
 
-bool operator== (const MPEZoneLayout& a, const MPEZoneLayout& b)
-{
-    if (a.getLowerZone() != b.getLowerZone())
-        return false;
-
-    if (a.getUpperZone() != b.getUpperZone())
-        return false;
-
-    return true;
-}
-
-bool operator!= (const MPEZoneLayout& a, const MPEZoneLayout& b)
-{
-    return ! (a == b);
-}
-
 template<>
 struct VariantConverter<LoopMode>
 {
