@@ -63,6 +63,12 @@ public:
         log.add ("Parameter debug log started");
     }
 
+    ~PluginDebugWindow() override
+    {
+        for (auto* p : audioProc.getParameters())
+            p->removeListener (this);
+    }
+
     void parameterValueChanged (int parameterIndex, float newValue) override
     {
         auto* param = audioProc.getParameters()[parameterIndex];
