@@ -2,17 +2,16 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2017 - ROLI Ltd.
+   Copyright (c) 2020 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
 
-   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
-   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
-   27th April 2017).
+   By using JUCE, you agree to the terms of both the JUCE 6 End-User License
+   Agreement and JUCE Privacy Policy (both effective as of the 16th June 2020).
 
-   End User License Agreement: www.juce.com/juce-5-licence
-   Privacy Policy: www.juce.com/juce-5-privacy-policy
+   End User License Agreement: www.juce.com/juce-6-licence
+   Privacy Policy: www.juce.com/juce-privacy-policy
 
    Or: You may also use this code under the terms of the GPL v3 (see
    www.gnu.org/licenses).
@@ -554,7 +553,7 @@ void ComponentTypeHandler::fillInMemberVariableDeclarations (GeneratedCode& code
     String clsName (component->getProperties() ["virtualName"].toString());
 
     if (clsName.isNotEmpty())
-        clsName = CodeHelpers::makeValidIdentifier (clsName, false, false, true);
+        clsName = build_tools::makeValidIdentifier (clsName, false, false, true);
     else
         clsName = getClassName (component);
 
@@ -592,7 +591,7 @@ void ComponentTypeHandler::fillInCreationCode (GeneratedCode& code, Component* c
     s << memberVariableName << ".reset (new ";
 
     if (virtualName.isNotEmpty())
-        s << CodeHelpers::makeValidIdentifier (virtualName, false, false, true);
+        s << build_tools::makeValidIdentifier (virtualName, false, false, true);
     else
         s << getClassName (component);
 
@@ -613,7 +612,7 @@ void ComponentTypeHandler::fillInCreationCode (GeneratedCode& code, Component* c
     s << "addAndMakeVisible (" << memberVariableName << ".get());\n";
 
 
-    if (SettableTooltipClient* ttc = dynamic_cast<SettableTooltipClient*> (component))
+    if (auto* ttc = dynamic_cast<SettableTooltipClient*> (component))
     {
         if (ttc->getTooltip().isNotEmpty())
         {
@@ -623,7 +622,7 @@ void ComponentTypeHandler::fillInCreationCode (GeneratedCode& code, Component* c
         }
     }
 
-    if (component->getExplicitFocusOrder() > 0)
+    if (component != nullptr && component->getExplicitFocusOrder() > 0)
         s << memberVariableName << "->setExplicitFocusOrder ("
           << component->getExplicitFocusOrder()
           << ");\n";
