@@ -2,17 +2,16 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2017 - ROLI Ltd.
+   Copyright (c) 2020 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
 
-   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
-   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
-   27th April 2017).
+   By using JUCE, you agree to the terms of both the JUCE 6 End-User License
+   Agreement and JUCE Privacy Policy (both effective as of the 16th June 2020).
 
-   End User License Agreement: www.juce.com/juce-5-licence
-   Privacy Policy: www.juce.com/juce-5-privacy-policy
+   End User License Agreement: www.juce.com/juce-6-licence
+   Privacy Policy: www.juce.com/juce-privacy-policy
 
    Or: You may also use this code under the terms of the GPL v3 (see
    www.gnu.org/licenses).
@@ -40,7 +39,7 @@ public:
 
     //==============================================================================
     ImageButtonHandler()
-        : ButtonHandler ("Image Button", "ImageButton", typeid (ImageButton), 150, 24)
+        : ButtonHandler ("Image Button", "juce::ImageButton", typeid (ImageButton), 150, 24)
     {
     }
 
@@ -164,9 +163,9 @@ public:
         const String resName (getImageResource (ib, role));
 
         if (resName.isEmpty())
-            return "Image()";
+            return "juce::Image()";
 
-        return "ImageCache::getFromMemory (" + resName + ", " + resName + "Size)";
+        return "juce::ImageCache::getFromMemory (" + resName + ", " + resName + "Size)";
     }
 
     //==============================================================================
@@ -465,7 +464,7 @@ public:
     }
 
     class ImageButtonColourProperty    : public JucerColourPropertyComponent,
-                                         public ChangeListener
+                                         private ChangeListener
     {
     public:
         ImageButtonColourProperty (ComponentLayout& layout_, ImageButton* const owner_,
@@ -478,29 +477,29 @@ public:
             layout_.getDocument()->addChangeListener (this);
         }
 
-        ~ImageButtonColourProperty()
+        ~ImageButtonColourProperty() override
         {
             layout.getDocument()->removeChangeListener (this);
         }
 
-        void setColour (Colour newColour)
+        void setColour (Colour newColour) override
         {
             setImageColour (layout, owner, role, newColour, true);
         }
 
-        Colour getColour() const
+        Colour getColour() const override
         {
             return getImageColour (owner, role);
         }
 
-        void resetToDefault() {}
+        void resetToDefault() override {}
 
-        void changeListenerCallback (ChangeBroadcaster*)
+    private:
+        void changeListenerCallback (ChangeBroadcaster*) override
         {
             refresh();
         }
 
-    private:
         ImageButton* const owner;
         ComponentLayout& layout;
         const ImageRole role;
