@@ -2,17 +2,16 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2017 - ROLI Ltd.
+   Copyright (c) 2020 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
 
-   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
-   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
-   27th April 2017).
+   By using JUCE, you agree to the terms of both the JUCE 6 End-User License
+   Agreement and JUCE Privacy Policy (both effective as of the 16th June 2020).
 
-   End User License Agreement: www.juce.com/juce-5-licence
-   Privacy Policy: www.juce.com/juce-5-privacy-policy
+   End User License Agreement: www.juce.com/juce-6-licence
+   Privacy Policy: www.juce.com/juce-privacy-policy
 
    Or: You may also use this code under the terms of the GPL v3 (see
    www.gnu.org/licenses).
@@ -32,7 +31,7 @@ class TabbedComponentHandler  : public ComponentTypeHandler
 {
 public:
     TabbedComponentHandler()
-        : ComponentTypeHandler ("Tabbed Component", "TabbedComponent", typeid (TabbedComponent), 200, 150)
+        : ComponentTypeHandler ("Tabbed Component", "juce::TabbedComponent", typeid (TabbedComponent), 200, 150)
     {}
 
     Component* createNewComponent (JucerDocument*) override
@@ -83,7 +82,7 @@ public:
 
         t->clearTabs();
 
-        forEachXmlChildElement (xml, e)
+        for (auto* e : xml.getChildIterator())
         {
             addNewTab (t);
             restoreTabState (t, t->getNumTabs() - 1, *e);
@@ -152,10 +151,10 @@ public:
 
         switch (t->getOrientation())
         {
-            case TabbedButtonBar::TabsAtTop:        return "TabbedButtonBar::TabsAtTop";
-            case TabbedButtonBar::TabsAtBottom:     return "TabbedButtonBar::TabsAtBottom";
-            case TabbedButtonBar::TabsAtLeft:       return "TabbedButtonBar::TabsAtLeft";
-            case TabbedButtonBar::TabsAtRight:      return "TabbedButtonBar::TabsAtRight";
+            case TabbedButtonBar::TabsAtTop:        return "juce::TabbedButtonBar::TabsAtTop";
+            case TabbedButtonBar::TabsAtBottom:     return "juce::TabbedButtonBar::TabsAtBottom";
+            case TabbedButtonBar::TabsAtLeft:       return "juce::TabbedButtonBar::TabsAtLeft";
+            case TabbedButtonBar::TabsAtRight:      return "juce::TabbedButtonBar::TabsAtRight";
             default:                                jassertfalse; break;
         }
 
@@ -531,7 +530,7 @@ private:
 
     //==============================================================================
     class TabDepthProperty  : public SliderPropertyComponent,
-                              public ChangeListener
+                              private ChangeListener
     {
     public:
         TabDepthProperty (TabbedComponent* comp, JucerDocument& doc)
@@ -542,12 +541,12 @@ private:
             document.addChangeListener (this);
         }
 
-        ~TabDepthProperty()
+        ~TabDepthProperty() override
         {
             document.removeChangeListener (this);
         }
 
-        void setValue (double newValue)
+        void setValue (double newValue) override
         {
             document.getUndoManager().undoCurrentTransactionOnly();
 
@@ -555,20 +554,20 @@ private:
                               "Change TabComponent tab depth");
         }
 
-        double getValue() const
+        double getValue() const override
         {
             return component->getTabBarDepth();
         }
 
-        void changeListenerCallback (ChangeBroadcaster*)
-        {
-            refresh();
-        }
-
         TabbedComponent* const component;
         JucerDocument& document;
 
     private:
+        void changeListenerCallback (ChangeBroadcaster*) override
+        {
+            refresh();
+        }
+
         class TabDepthChangeAction  : public ComponentUndoableAction<TabbedComponent>
         {
         public:
@@ -673,13 +672,13 @@ private:
                 m.addItem (i + 1, "Delete tab " + String (i)
                                     + ": \"" + names[i] + "\"");
 
-            const int r = m.showAt (this);
-
-            if (r > 0)
+            PopupMenu::Options options{};
+            m.showMenuAsync (PopupMenu::Options().withTargetComponent (this), [this] (int r)
             {
-                document.perform (new RemoveTabAction (component, *document.getComponentLayout(), r - 1),
-                                  "Remove a tab");
-            }
+                if (r > 0)
+                    document.perform (new RemoveTabAction (component, *document.getComponentLayout(), r - 1),
+                                      "Remove a tab");
+            });
         }
 
         String getButtonText() const
@@ -925,7 +924,7 @@ private:
 
     //==============================================================================
     class TabJucerFileProperty   : public FilePropertyComponent,
-                                   public ChangeListener
+                                   private ChangeListener
     {
     public:
         TabJucerFileProperty (TabbedComponent* const comp, JucerDocument& doc, const int tabIndex_)
@@ -937,13 +936,13 @@ private:
             document.addChangeListener (this);
         }
 
-        ~TabJucerFileProperty()
+        ~TabJucerFileProperty() override
         {
             document.removeChangeListener (this);
         }
 
         //==============================================================================
-        void setFile (const File& newFile)
+        void setFile (const File& newFile) override
         {
             document.perform (new JucerCompFileChangeAction (component, *document.getComponentLayout(), tabIndex,
                                                              newFile.getRelativePathFrom (document.getCppFile().getParentDirectory())
@@ -951,14 +950,14 @@ private:
                               "Change tab component file");
         }
 
-        File getFile() const
+        File getFile() const override
         {
             return document.getCppFile().getSiblingFile (getTabJucerFile (component, tabIndex));
         }
 
-        void changeListenerCallback (ChangeBroadcaster*)     { refresh(); }
-
     private:
+        void changeListenerCallback (ChangeBroadcaster*) override { refresh(); }
+
         TabbedComponent* const component;
         JucerDocument& document;
         int tabIndex;
@@ -1132,11 +1131,13 @@ private:
             m.addItem (1, "Move this tab up", tabIndex > 0);
             m.addItem (2, "Move this tab down", tabIndex < totalNumTabs - 1);
 
-            const int r = m.showAt (this);
-
-            if (r != 0)
-                document.perform (new MoveTabAction (component, *document.getComponentLayout(), tabIndex, tabIndex + (r == 2 ? 1 : -1)),
-                                  "Move a tab");
+            PopupMenu::Options options{};
+            m.showMenuAsync (PopupMenu::Options().withTargetComponent (this), [this] (int r)
+            {
+                if (r != 0)
+                    document.perform (new MoveTabAction (component, *document.getComponentLayout(), tabIndex, tabIndex + (r == 2 ? 1 : -1)),
+                                      "Move a tab");
+            });
         }
 
         String getButtonText() const
