@@ -2,17 +2,16 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2017 - ROLI Ltd.
+   Copyright (c) 2020 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
 
-   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
-   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
-   27th April 2017).
+   By using JUCE, you agree to the terms of both the JUCE 6 End-User License
+   Agreement and JUCE Privacy Policy (both effective as of the 16th June 2020).
 
-   End User License Agreement: www.juce.com/juce-5-licence
-   Privacy Policy: www.juce.com/juce-5-privacy-policy
+   End User License Agreement: www.juce.com/juce-6-licence
+   Privacy Policy: www.juce.com/juce-privacy-policy
 
    Or: You may also use this code under the terms of the GPL v3 (see
    www.gnu.org/licenses).
@@ -78,20 +77,21 @@ private:
     {
         showCorrectTab();
 
-        PaintElementPath* const path = getElement();
-        jassert (path != nullptr);
+        if (auto* const path = getElement())
+        {
+            if (auto* const p = path->getPoint (index))
+            {
+                const auto typeChanged = (p->type != value.type);
+                *p = value;
+                p->owner = path;
 
-        PathPoint* const p = path->getPoint (index);
-        jassert (p != nullptr);
+                if (typeChanged)
+                    path->pointListChanged();
 
-        const bool typeChanged = (p->type != value.type);
-        *p = value;
-        p->owner = path;
+                path->changed();
+            }
+        }
 
-        if (typeChanged)
-            path->pointListChanged();
-
-        path->changed();
         return true;
     }
 };
@@ -99,10 +99,10 @@ private:
 
 //==============================================================================
 class PathWindingModeProperty    : public ChoicePropertyComponent,
-                                   public ChangeListener
+                                   private ChangeListener
 {
 public:
-    PathWindingModeProperty (PaintElementPath* const owner_)
+    explicit PathWindingModeProperty (PaintElementPath* const owner_)
         : ChoicePropertyComponent ("winding rule"),
           owner (owner_)
     {
@@ -112,17 +112,17 @@ public:
         owner->getDocument()->addChangeListener (this);
     }
 
-    ~PathWindingModeProperty()
+    ~PathWindingModeProperty() override
     {
         owner->getDocument()->removeChangeListener (this);
     }
 
-    void setIndex (int newIndex)            { owner->setNonZeroWinding (newIndex == 0, true); }
-    int getIndex() const                    { return owner->isNonZeroWinding() ? 0 : 1; }
-
-    void changeListenerCallback (ChangeBroadcaster*)     { refresh(); }
+    void setIndex (int newIndex) override   { owner->setNonZeroWinding (newIndex == 0, true); }
+    int getIndex() const         override   { return owner->isNonZeroWinding() ? 0 : 1; }
 
 private:
+    void changeListenerCallback (ChangeBroadcaster*) override { refresh(); }
+
     PaintElementPath* const owner;
 };
 
@@ -377,7 +377,7 @@ void PaintElementPath::fillInGeneratedCode (GeneratedCode& code, String& paintMe
     const ComponentLayout* layout = code.document->getComponentLayout();
 
     code.privateMemberDeclarations
-        << "Path " << pathVariable << ";\n";
+        << "juce::Path " << pathVariable << ";\n";
 
     String r;
     bool somePointsAreRelative = false;
@@ -455,14 +455,14 @@ void PaintElementPath::fillInGeneratedCode (GeneratedCode& code, String& paintMe
     {
         s << "    ";
         fillType.fillInGeneratedCode ("fill", zero, code, s);
-        s << "    g.fillPath (" << pathVariable << ", AffineTransform::translation(x, y));\n";
+        s << "    g.fillPath (" << pathVariable << ", juce::AffineTransform::translation(x, y));\n";
     }
 
     if (isStrokePresent && ! strokeType.isInvisible())
     {
         s << "    ";
         strokeType.fill.fillInGeneratedCode ("stroke", zero, code, s);
-        s << "    g.strokePath (" << pathVariable << ", " << strokeType.getPathStrokeCode() << ", AffineTransform::translation(x, y));\n";
+        s << "    g.strokePath (" << pathVariable << ", " << strokeType.getPathStrokeCode() << ", juce::AffineTransform::translation(x, y));\n";
     }
 
     s << "}\n\n";
@@ -850,14 +850,15 @@ public:
     {
         showCorrectTab();
 
-        PaintElementPath* const path = getElement();
-        jassert (path != nullptr);
+        if (auto* const path = getElement())
+        {
+            if (auto* const p = path->addPoint (pointIndexToAddItAfter, false))
+            {
+                indexAdded = path->indexOfPoint (p);
+                jassert (indexAdded >= 0);
+            }
+        }
 
-        PathPoint* const p = path->addPoint (pointIndexToAddItAfter, false);
-        jassert (p != nullptr);
-
-        indexAdded = path->indexOfPoint (p);
-        jassert (indexAdded >= 0);
         return true;
     }
 
@@ -1013,6 +1014,12 @@ bool PaintElementPath::getPoint (int index, int pointNumber, double& x, double&
         return false;
     }
 
+    if (pointNumber >= PathPoint::maxRects)
+    {
+        jassertfalse;
+        return false;
+    }
+
     jassert (pointNumber < 3 || p->type == Path::Iterator::cubicTo);
     jassert (pointNumber < 2 || p->type == Path::Iterator::cubicTo || p->type == Path::Iterator::quadraticTo);
 
@@ -1118,6 +1125,12 @@ void PaintElementPath::movePoint (int index, int pointNumber,
         jassert (pointNumber < 3 || p->type == Path::Iterator::cubicTo);
         jassert (pointNumber < 2 || p->type == Path::Iterator::cubicTo || p->type == Path::Iterator::quadraticTo);
 
+        if (pointNumber >= PathPoint::maxRects)
+        {
+            jassertfalse;
+            return;
+        }
+
         RelativePositionedRectangle& pr = newPoint.pos [pointNumber];
 
         double x, y, w, h;
@@ -1138,6 +1151,12 @@ void PaintElementPath::movePoint (int index, int pointNumber,
 
 RelativePositionedRectangle PaintElementPath::getPoint (int index, int pointNumber) const
 {
+    if (pointNumber >= PathPoint::maxRects)
+    {
+        jassertfalse;
+        return RelativePositionedRectangle();
+    }
+
     if (PathPoint* const p = points [index])
     {
         jassert (pointNumber < 3 || p->type == Path::Iterator::cubicTo);
@@ -1152,6 +1171,12 @@ RelativePositionedRectangle PaintElementPath::getPoint (int index, int pointNumb
 
 void PaintElementPath::setPoint (int index, int pointNumber, const RelativePositionedRectangle& newPos, const bool undoable)
 {
+    if (pointNumber >= PathPoint::maxRects)
+    {
+        jassertfalse;
+        return;
+    }
+
     if (PathPoint* const p = points [index])
     {
         PathPoint newPoint (*p);
@@ -1182,7 +1207,7 @@ void PaintElementPath::setPoint (int index, int pointNumber, const RelativePosit
 
 //==============================================================================
 class PathPointTypeProperty : public ChoicePropertyComponent,
-                              public ChangeListener
+                              private ChangeListener
 {
 public:
     PathPointTypeProperty (PaintElementPath* const owner_,
@@ -1199,12 +1224,12 @@ public:
         owner->getDocument()->addChangeListener (this);
     }
 
-    ~PathPointTypeProperty()
+    ~PathPointTypeProperty() override
     {
         owner->getDocument()->removeChangeListener (this);
     }
 
-    void setIndex (int newIndex)
+    void setIndex (int newIndex) override
     {
         Path::Iterator::PathElementType type = Path::Iterator::startNewSubPath;
 
@@ -1221,30 +1246,30 @@ public:
         owner->getPoint (index)->changePointType (type, area, true);
     }
 
-    int getIndex() const
+    int getIndex() const override
     {
-        const PathPoint* const p = owner->getPoint (index);
-        jassert (p != nullptr);
-
-        switch (p->type)
+        if (const auto* const p = owner->getPoint (index))
         {
-            case Path::Iterator::startNewSubPath:   return 0;
-            case Path::Iterator::lineTo:            return 1;
-            case Path::Iterator::quadraticTo:       return 2;
-            case Path::Iterator::cubicTo:           return 3;
-            case Path::Iterator::closePath:         break;
-            default:                                jassertfalse; break;
+            switch (p->type)
+            {
+                case Path::Iterator::startNewSubPath:   return 0;
+                case Path::Iterator::lineTo:            return 1;
+                case Path::Iterator::quadraticTo:       return 2;
+                case Path::Iterator::cubicTo:           return 3;
+                case Path::Iterator::closePath:         break;
+                default:                                jassertfalse; break;
+            }
         }
 
         return 0;
     }
 
-    void changeListenerCallback (ChangeBroadcaster*)
+private:
+    void changeListenerCallback (ChangeBroadcaster*) override
     {
         refresh();
     }
 
-private:
     PaintElementPath* const owner;
     const int index;
 };
