@@ -2,17 +2,16 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2017 - ROLI Ltd.
+   Copyright (c) 2020 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
 
-   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
-   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
-   27th April 2017).
+   By using JUCE, you agree to the terms of both the JUCE 6 End-User License
+   Agreement and JUCE Privacy Policy (both effective as of the 16th June 2020).
 
-   End User License Agreement: www.juce.com/juce-5-licence
-   Privacy Policy: www.juce.com/juce-5-privacy-policy
+   End User License Agreement: www.juce.com/juce-6-licence
+   Privacy Policy: www.juce.com/juce-privacy-policy
 
    Or: You may also use this code under the terms of the GPL v3 (see
    www.gnu.org/licenses).
@@ -71,7 +70,7 @@ public:
 
         if (multipleSelected)
             return;
-        
+
         props.add (new TextProperty (this));
         props.add (new FontNameProperty (this));
         props.add (new FontStyleProperty (this));
@@ -89,7 +88,7 @@ public:
             positionToCode (position, code.document->getComponentLayout(), x, y, w, h);
             r << "{\n"
               << "    int x = " << x << ", y = " << y << ", width = " << w << ", height = " << h << ";\n"
-              << "    String text (" << quotedString (text, code.shouldUseTransMacro()) << ");\n"
+              << "    juce::String text (" << quotedString (text, code.shouldUseTransMacro()) << ");\n"
               << "    " << fillType.generateVariablesCode ("fill")
               << "    //[UserPaintCustomArguments] Customize the painting arguments here..\n"
               << customPaintCode
@@ -100,7 +99,7 @@ public:
               << "    g.drawText (text, x, y, width, height,\n"
               << "                " << CodeHelpers::justificationToCode (justification) << ", true);\n"
               << "}\n\n";
-            
+
             paintMethodCode += r;
         }
     }
@@ -108,7 +107,7 @@ public:
     void applyCustomPaintSnippets (StringArray& snippets) override
     {
         customPaintCode.clear();
-        
+
         if (! snippets.isEmpty() && ! fillType.isInvisible())
         {
             customPaintCode = snippets[0];
@@ -390,12 +389,12 @@ private:
     String typefaceName;
     Justification justification;
     String customPaintCode;
-    
+
     Array <Justification> justificationTypes;
 
     //==============================================================================
     class TextProperty  : public TextPropertyComponent,
-                          public ChangeListener
+                          private juce::ChangeListener
     {
     public:
         TextProperty (PaintElementText* const e)
@@ -413,15 +412,15 @@ private:
         void setText (const String& newText) override    { element->setText (newText, true); }
         String getText() const override                  { return element->getText(); }
 
+    private:
         void changeListenerCallback (ChangeBroadcaster*) override     { refresh(); }
 
-    private:
         PaintElementText* const element;
     };
 
     //==============================================================================
     class FontNameProperty  : public FontPropertyComponent,
-                              public ChangeListener
+                              private juce::ChangeListener
     {
     public:
         FontNameProperty (PaintElementText* const e)
@@ -439,15 +438,15 @@ private:
         void setTypefaceName (const String& newFontName)    { element->setTypefaceName (newFontName, true); }
         String getTypefaceName() const                      { return element->getTypefaceName(); }
 
+    private:
         void changeListenerCallback (ChangeBroadcaster*)                 { refresh(); }
 
-    private:
         PaintElementText* const element;
     };
 
     //==============================================================================
     class FontStyleProperty  : public ChoicePropertyComponent,
-                               public ChangeListener
+                               private juce::ChangeListener
     {
     public:
         FontStyleProperty (PaintElementText* const e)
@@ -526,18 +525,18 @@ private:
             return typefaceIndex;
         }
 
+    private:
         void changeListenerCallback (ChangeBroadcaster*)
         {
             updateStylesList (element->getTypefaceName());
         }
 
-    private:
         PaintElementText* const element;
     };
 
     //==============================================================================
     class FontSizeProperty  : public SliderPropertyComponent,
-                              public ChangeListener
+                              private juce::ChangeListener
     {
     public:
         FontSizeProperty (PaintElementText* const e)
@@ -547,12 +546,12 @@ private:
             element->getDocument()->addChangeListener (this);
         }
 
-        ~FontSizeProperty()
+        ~FontSizeProperty() override
         {
             element->getDocument()->removeChangeListener (this);
         }
 
-        void setValue (double newValue)
+        void setValue (double newValue) override
         {
             element->getDocument()->getUndoManager().undoCurrentTransactionOnly();
 
@@ -562,20 +561,20 @@ private:
             element->setFont (f, true);
         }
 
-        double getValue() const
+        double getValue() const override
         {
             return element->getFont().getHeight();
         }
 
-        void changeListenerCallback (ChangeBroadcaster*)     { refresh(); }
-
     private:
+        void changeListenerCallback (ChangeBroadcaster*) override { refresh(); }
+
         PaintElementText* const element;
     };
 
     //==============================================================================
     class FontKerningProperty  : public SliderPropertyComponent,
-                                 public ChangeListener
+                                 private juce::ChangeListener
     {
     public:
         FontKerningProperty (PaintElementText* const e)
@@ -585,12 +584,12 @@ private:
             element->getDocument()->addChangeListener (this);
         }
 
-        ~FontKerningProperty()
+        ~FontKerningProperty() override
         {
             element->getDocument()->removeChangeListener (this);
         }
 
-        void setValue (double newValue)
+        void setValue (double newValue) override
         {
             element->getDocument()->getUndoManager().undoCurrentTransactionOnly();
 
@@ -600,23 +599,23 @@ private:
             element->setFont (f, true);
         }
 
-        double getValue() const
+        double getValue() const override
         {
             return element->getFont().getExtraKerningFactor();
         }
 
-        void changeListenerCallback (ChangeBroadcaster*)
+    private:
+        void changeListenerCallback (ChangeBroadcaster*) override
         {
             refresh();
         }
 
-    private:
         PaintElementText* const element;
     };
 
     //==============================================================================
     class TextJustificationProperty  : public JustificationProperty,
-                                       public ChangeListener
+                                       private juce::ChangeListener
     {
     public:
         TextJustificationProperty (PaintElementText* const e)
@@ -626,24 +625,24 @@ private:
             element->getDocument()->addChangeListener (this);
         }
 
-        ~TextJustificationProperty()
+        ~TextJustificationProperty() override
         {
             element->getDocument()->removeChangeListener (this);
         }
 
-        void setJustification (Justification newJustification)
+        void setJustification (Justification newJustification) override
         {
             element->setJustification (newJustification, true);
         }
 
-        Justification getJustification() const
+        Justification getJustification() const override
         {
             return element->getJustification();
         }
 
-        void changeListenerCallback (ChangeBroadcaster*)     { refresh(); }
-
     private:
+        void changeListenerCallback (ChangeBroadcaster*) override { refresh(); }
+
         PaintElementText* const element;
     };
 
