@@ -2,17 +2,16 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2017 - ROLI Ltd.
+   Copyright (c) 2020 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
 
-   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
-   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
-   27th April 2017).
+   By using JUCE, you agree to the terms of both the JUCE 6 End-User License
+   Agreement and JUCE Privacy Policy (both effective as of the 16th June 2020).
 
-   End User License Agreement: www.juce.com/juce-5-licence
-   Privacy Policy: www.juce.com/juce-5-privacy-policy
+   End User License Agreement: www.juce.com/juce-6-licence
+   Privacy Policy: www.juce.com/juce-privacy-policy
 
    Or: You may also use this code under the terms of the GPL v3 (see
    www.gnu.org/licenses).
@@ -75,8 +74,8 @@ public:
     */
     PositionedRectangle() noexcept
         : x (0.0), y (0.0), w (0.0), h (0.0),
-          xMode (anchorAtLeftOrTop | absoluteFromParentTopLeft),
-          yMode (anchorAtLeftOrTop | absoluteFromParentTopLeft),
+          xMode ((int) anchorAtLeftOrTop | (int) absoluteFromParentTopLeft),
+          yMode ((int) anchorAtLeftOrTop | (int) absoluteFromParentTopLeft),
           wMode (absoluteSize), hMode (absoluteSize)
     {
     }
@@ -284,23 +283,23 @@ public:
                    const SizeMode widthMode, const SizeMode heightMode,
                    const Rectangle<int>& target) noexcept
     {
-        if (xMode != (xAnchor | xMode_) || wMode != widthMode)
+        if (xMode != ((int) xAnchor | (int) xMode_) || wMode != widthMode)
         {
             double tx, tw;
             applyPosAndSize (tx, tw, x, w, xMode, wMode, target.getX(), target.getWidth());
 
-            xMode = (uint8) (xAnchor | xMode_);
+            xMode = (uint8) ((int) xAnchor | (int) xMode_);
             wMode = (uint8) widthMode;
 
             updatePosAndSize (x, w, tx, tw, xMode, wMode, target.getX(), target.getWidth());
         }
 
-        if (yMode != (yAnchor | yMode_) || hMode != heightMode)
+        if (yMode != ((int) yAnchor | (int) yMode_) || hMode != heightMode)
         {
             double ty, th;
             applyPosAndSize (ty, th, y, h, yMode, hMode, target.getY(), target.getHeight());
 
-            yMode = (uint8) (yAnchor | yMode_);
+            yMode = (uint8) ((int) yAnchor | (int) yMode_);
             hMode = (uint8) heightMode;
 
             updatePosAndSize (y, h, ty, th, yMode, hMode, target.getY(), target.getHeight());
@@ -740,7 +739,7 @@ struct RelativePositionedRectangle
     {
         double x, y, w, h;
         getRectangleDouble (x, y, w, h, parentArea, layout);
-        return Point<float> ((float) x, (float) y);
+        return { (float) x, (float) y };
     }
 
     void getXY (double& x, double& y,
