@@ -2,17 +2,16 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2017 - ROLI Ltd.
+   Copyright (c) 2020 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
 
-   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
-   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
-   27th April 2017).
+   By using JUCE, you agree to the terms of both the JUCE 6 End-User License
+   Agreement and JUCE Privacy Policy (both effective as of the 16th June 2020).
 
-   End User License Agreement: www.juce.com/juce-5-licence
-   Privacy Policy: www.juce.com/juce-5-privacy-policy
+   End User License Agreement: www.juce.com/juce-6-licence
+   Privacy Policy: www.juce.com/juce-privacy-policy
 
    Or: You may also use this code under the terms of the GPL v3 (see
    www.gnu.org/licenses).
@@ -29,7 +28,7 @@
 
 //==============================================================================
 class FileTreeItemBase   : public JucerTreeViewBase,
-                           public ValueTree::Listener
+                           private ValueTree::Listener
 {
 public:
     FileTreeItemBase (const Project::Item& projectItem)
@@ -85,40 +84,26 @@ public:
             }
         }
 
-        if (filesToTrash.size() > 0)
+        WeakReference<FileTreeItemBase> treeRootItem { dynamic_cast<FileTreeItemBase*> (tree->getRootItem()) };
+
+        if (treeRootItem == nullptr)
         {
-            String fileList;
-            auto maxFilesToList = 10;
-            for (auto i = jmin (maxFilesToList, filesToTrash.size()); --i >= 0;)
-                fileList << filesToTrash.getUnchecked(i).getFullPathName() << "\n";
-
-            if (filesToTrash.size() > maxFilesToList)
-                fileList << "\n...plus " << (filesToTrash.size() - maxFilesToList) << " more files...";
-
-            auto r = AlertWindow::showYesNoCancelBox (AlertWindow::NoIcon, "Delete Project Items",
-                                                      "As well as removing the selected item(s) from the project, do you also want to move their files to the trash:\n\n"
-                                                           + fileList,
-                                                      "Just remove references",
-                                                      "Also move files to Trash",
-                                                      "Cancel",
-                                                      tree->getTopLevelComponent());
-
-            if (r == 0)
-                return;
-
-            if (r != 2)
-                filesToTrash.clear();
+            jassertfalse;
+            return;
         }
 
-        if (auto* treeRootItem = dynamic_cast<FileTreeItemBase*> (tree->getRootItem()))
+        auto doDelete = [treeRootItem, itemsToRemove] (const Array<File>& fsToTrash)
         {
+            if (treeRootItem == nullptr)
+                return;
+
             auto& om = ProjucerApplication::getApp().openDocumentManager;
 
-            for (auto i = filesToTrash.size(); --i >= 0;)
+            for (auto i = fsToTrash.size(); --i >= 0;)
             {
-                auto f = filesToTrash.getUnchecked(i);
+                auto f = fsToTrash.getUnchecked(i);
 
-                om.closeFile (f, false);
+                om.closeFileWithoutSaving (f);
 
                 if (! f.moveToTrash())
                 {
@@ -132,20 +117,53 @@ public:
                 {
                     if (auto* pcc = treeRootItem->getProjectContentComponent())
                     {
-                        if (auto* fileInfoComp = dynamic_cast<FileGroupInformationComponent*> (pcc->getEditorComponentContent()))
+                        if (auto* fileInfoComp = dynamic_cast<FileGroupInformationComponent*> (pcc->getEditorComponent()))
                             if (fileInfoComp->getGroupPath() == itemToRemove->getFile().getFullPathName())
                                 pcc->hideEditor();
                     }
 
-                    om.closeFile (itemToRemove->getFile(), false);
+                    om.closeFileWithoutSaving (itemToRemove->getFile());
                     itemToRemove->deleteItem();
                 }
             }
-        }
-        else
+        };
+
+        if (! filesToTrash.isEmpty())
         {
-            jassertfalse;
+            String fileList;
+            auto maxFilesToList = 10;
+            for (auto i = jmin (maxFilesToList, filesToTrash.size()); --i >= 0;)
+                fileList << filesToTrash.getUnchecked(i).getFullPathName() << "\n";
+
+            if (filesToTrash.size() > maxFilesToList)
+                fileList << "\n...plus " << (filesToTrash.size() - maxFilesToList) << " more files...";
+
+            AlertWindow::showYesNoCancelBox (MessageBoxIconType::NoIcon,
+                                             "Delete Project Items",
+                                             "As well as removing the selected item(s) from the project, do you also want to move their files to the trash:\n\n"
+                                                  + fileList,
+                                             "Just remove references",
+                                             "Also move files to Trash",
+                                             "Cancel",
+                                             tree->getTopLevelComponent(),
+                                             ModalCallbackFunction::create ([treeRootItem, filesToTrash, doDelete] (int r) mutable
+            {
+                if (treeRootItem == nullptr)
+                    return;
+
+                if (r == 0)
+                    return;
+
+                if (r != 2)
+                    filesToTrash.clear();
+
+                doDelete (filesToTrash);
+            }));
+
+            return;
         }
+
+        doDelete (filesToTrash);
     }
 
     virtual void revealInFinder() const
@@ -156,17 +174,24 @@ public:
     virtual void browseToAddExistingFiles()
     {
         auto location = item.isGroup() ? item.determineGroupFolder() : getFile();
-        FileChooser fc ("Add Files to Jucer Project", location, {});
+        chooser = std::make_unique<FileChooser> ("Add Files to Jucer Project", location, "");
+        auto flags = FileBrowserComponent::openMode
+                   | FileBrowserComponent::canSelectFiles
+                   | FileBrowserComponent::canSelectDirectories
+                   | FileBrowserComponent::canSelectMultipleItems;
 
-        if (fc.browseForMultipleFilesOrDirectories())
+        chooser->launchAsync (flags, [this] (const FileChooser& fc)
         {
+            if (fc.getResults().isEmpty())
+                return;
+
             StringArray files;
 
             for (int i = 0; i < fc.getResults().size(); ++i)
                 files.add (fc.getResults().getReference(i).getFullPathName());
 
             addFilesRetainingSortOrder (files);
-        }
+        });
     }
 
     virtual void checkFileStatus()  // (recursive)
@@ -193,17 +218,17 @@ public:
             p->addFilesRetainingSortOrder (files);
     }
 
-    virtual void moveSelectedItemsTo (OwnedArray <Project::Item>&, int /*insertIndex*/)
+    virtual void moveSelectedItemsTo (OwnedArray<Project::Item>&, int /*insertIndex*/)
     {
         jassertfalse;
     }
 
-    void showMultiSelectionPopupMenu() override
+    void showMultiSelectionPopupMenu (Point<int> p) override
     {
         PopupMenu m;
         m.addItem (1, "Delete");
 
-        m.showMenuAsync (PopupMenu::Options(),
+        m.showMenuAsync (PopupMenu::Options().withTargetScreenArea ({ p.x, p.y, 1, 1 }),
                          ModalCallbackFunction::create (treeViewMultiSelectItemChosen, this));
     }
 
@@ -235,17 +260,6 @@ public:
         return nullptr;
     }
 
-    //==============================================================================
-    void valueTreePropertyChanged (ValueTree& tree, const Identifier&) override
-    {
-        if (tree == item.state)
-            repaintItem();
-    }
-
-    void valueTreeChildAdded (ValueTree& parentTree, ValueTree&) override         { treeChildrenChanged (parentTree); }
-    void valueTreeChildRemoved (ValueTree& parentTree, ValueTree&, int) override  { treeChildrenChanged (parentTree); }
-    void valueTreeChildOrderChanged (ValueTree& parentTree, int, int) override    { treeChildrenChanged (parentTree); }
-
     //==============================================================================
     bool mightContainSubItems() override                { return item.getNumChildren() > 0; }
     String getUniqueName() const override               { jassert (item.getID().isNotEmpty()); return item.getID(); }
@@ -281,7 +295,7 @@ public:
     void filesDropped (const StringArray& files, int insertIndex) override
     {
         if (files.size() == 1 && File (files[0]).hasFileExtension (Project::projectFileExtension))
-            ProjucerApplication::getApp().openFile (files[0]);
+            ProjucerApplication::getApp().openFile (files[0], [] (bool) {});
         else
             addFilesAtIndex (files, insertIndex);
     }
@@ -348,6 +362,17 @@ public:
     Project::Item item;
 
 protected:
+    //==============================================================================
+    void valueTreePropertyChanged (ValueTree& tree, const Identifier&) override
+    {
+        if (tree == item.state)
+            repaintItem();
+    }
+
+    void valueTreeChildAdded (ValueTree& parentTree, ValueTree&) override         { treeChildrenChanged (parentTree); }
+    void valueTreeChildRemoved (ValueTree& parentTree, ValueTree&, int) override  { treeChildrenChanged (parentTree); }
+    void valueTreeChildOrderChanged (ValueTree& parentTree, int, int) override    { treeChildrenChanged (parentTree); }
+
     bool isFileMissing;
 
     virtual FileTreeItemBase* createSubItem (const Project::Item& node) = 0;
@@ -445,6 +470,11 @@ protected:
 
         return -1;
     }
+
+private:
+    std::unique_ptr<FileChooser> chooser;
+
+    JUCE_DECLARE_WEAK_REFERENCEABLE (FileTreeItemBase)
 };
 
 //==============================================================================
@@ -457,7 +487,7 @@ public:
     }
 
     bool acceptsFileDrop (const StringArray&) const override             { return false; }
-    bool acceptsDragItems (const OwnedArray <Project::Item>&) override   { return false; }
+    bool acceptsDragItems (const OwnedArray<Project::Item>&) override    { return false; }
 
     String getDisplayName() const override
     {
@@ -491,8 +521,9 @@ public:
     {
         if (newName != File::createLegalFileName (newName))
         {
-            AlertWindow::showMessageBox (AlertWindow::WarningIcon, "File Rename",
-                                         "That filename contained some illegal characters!");
+            AlertWindow::showMessageBoxAsync (MessageBoxIconType::WarningIcon,
+                                              "File Rename",
+                                              "That filename contained some illegal characters!");
             triggerAsyncRename (item);
             return;
         }
@@ -507,30 +538,42 @@ public:
 
             if (correspondingItem.isValid())
             {
-                if (AlertWindow::showOkCancelBox (AlertWindow::NoIcon, "File Rename",
-                                                  "Do you also want to rename the corresponding file \"" + correspondingFile.getFileName()
-                                                    + "\" to match?"))
+                AlertWindow::showOkCancelBox (MessageBoxIconType::NoIcon,
+                                              "File Rename",
+                                              "Do you also want to rename the corresponding file \"" + correspondingFile.getFileName() + "\" to match?",
+                                              {},
+                                              {},
+                                              nullptr,
+                                              ModalCallbackFunction::create ([parent = WeakReference<SourceFileItem> { this },
+                                                                              oldFile, newFile, correspondingFile, correspondingItem] (int result) mutable
                 {
-                    if (! item.renameFile (newFile))
+                    if (parent == nullptr || result == 0)
+                        return;
+
+                    if (! parent->item.renameFile (newFile))
                     {
-                        AlertWindow::showMessageBox (AlertWindow::WarningIcon, "File Rename",
-                                                     "Failed to rename \"" + oldFile.getFullPathName() + "\"!\n\nCheck your file permissions!");
+                        AlertWindow::showMessageBoxAsync (MessageBoxIconType::WarningIcon,
+                                                          "File Rename",
+                                                          "Failed to rename \"" + oldFile.getFullPathName() + "\"!\n\nCheck your file permissions!");
                         return;
                     }
 
                     if (! correspondingItem.renameFile (newFile.withFileExtension (correspondingFile.getFileExtension())))
                     {
-                        AlertWindow::showMessageBox (AlertWindow::WarningIcon, "File Rename",
-                                                     "Failed to rename \"" + correspondingFile.getFullPathName() + "\"!\n\nCheck your file permissions!");
+                        AlertWindow::showMessageBoxAsync (MessageBoxIconType::WarningIcon,
+                                                          "File Rename",
+                                                          "Failed to rename \"" + correspondingFile.getFullPathName() + "\"!\n\nCheck your file permissions!");
                     }
-                }
+
+                }));
             }
         }
 
         if (! item.renameFile (newFile))
         {
-            AlertWindow::showMessageBox (AlertWindow::WarningIcon, "File Rename",
-                                         "Failed to rename the file!\n\nCheck your file permissions!");
+            AlertWindow::showMessageBoxAsync (MessageBoxIconType::WarningIcon,
+                                              "File Rename",
+                                              "Failed to rename the file!\n\nCheck your file permissions!");
         }
     }
 
@@ -549,7 +592,7 @@ public:
                 pcc->showEditorForFile (f, false);
     }
 
-    void showPopupMenu() override
+    void showPopupMenu (Point<int> p) override
     {
         PopupMenu m;
 
@@ -564,39 +607,35 @@ public:
         m.addItem (4, "Rename File...");
         m.addSeparator();
 
-        if (auto* group = dynamic_cast<GroupItem*> (getParentItem()))
-        {
-            if (group->isRoot())
-            {
-                m.addItem (5, "Binary Resource", true, item.shouldBeAddedToBinaryResources());
-                m.addItem (6, "Xcode Resource",  true, item.shouldBeAddedToXcodeResources());
-                m.addItem (7, "Compile",         true, item.shouldBeCompiled());
-                m.addSeparator();
-            }
-        }
+        m.addItem (5, "Binary Resource", true, item.shouldBeAddedToBinaryResources());
+        m.addItem (6, "Xcode Resource", true, item.shouldBeAddedToXcodeResources());
+        m.addItem (7, "Compile", item.isSourceFile(), item.shouldBeCompiled());
+        m.addItem (8, "Skip PCH", item.shouldBeCompiled(), item.shouldSkipPCH());
+        m.addSeparator();
 
         m.addItem (3, "Delete");
 
-        launchPopupMenu (m);
+        launchPopupMenu (m, p);
     }
 
-    void showAddMenu() override
+    void showAddMenu (Point<int> p) override
     {
         if (auto* group = dynamic_cast<GroupItem*> (getParentItem()))
-            group->showAddMenu();
+            group->showAddMenu (p);
     }
 
     void handlePopupMenuResult (int resultCode) override
     {
         switch (resultCode)
         {
-            case 1:     getFile().startAsProcess(); break;
-            case 2:     revealInFinder(); break;
-            case 3:     deleteAllSelectedItems(); break;
-            case 4:     triggerAsyncRename (item); break;
-            case 5:     item.getShouldAddToBinaryResourcesValue().setValue (! item.shouldBeAddedToBinaryResources()); break;
-            case 6:     item.getShouldAddToXcodeResourcesValue().setValue (! item.shouldBeAddedToXcodeResources()); break;
-            case 7:     item.getShouldCompileValue().setValue (! item.shouldBeCompiled()); break;
+            case 1:  getFile().startAsProcess(); break;
+            case 2:  revealInFinder(); break;
+            case 3:  deleteAllSelectedItems(); break;
+            case 4:  triggerAsyncRename (item); break;
+            case 5:  item.getShouldAddToBinaryResourcesValue().setValue (! item.shouldBeAddedToBinaryResources()); break;
+            case 6:  item.getShouldAddToXcodeResourcesValue().setValue (! item.shouldBeAddedToXcodeResources()); break;
+            case 7:  item.getShouldCompileValue().setValue (! item.shouldBeCompiled()); break;
+            case 8:  item.getShouldSkipPCHValue().setValue (! item.shouldSkipPCH()); break;
 
             default:
                 if (auto* parentGroup = dynamic_cast<GroupItem*> (getParentProjectItem()))
@@ -605,6 +644,8 @@ public:
                 break;
         }
     }
+
+    JUCE_DECLARE_WEAK_REFERENCEABLE (SourceFileItem)
 };
 
 //==============================================================================
@@ -701,7 +742,7 @@ public:
     void showDocument() override
     {
         if (auto* pcc = getProjectContentComponent())
-            pcc->setEditorComponent (new FileGroupInformationComponent (item), nullptr);
+            pcc->setScrollableEditorComponent (std::make_unique<FileGroupInformationComponent> (item));
     }
 
     static void openAllGroups (TreeViewItem* root)
@@ -736,7 +777,7 @@ public:
             setFilesToCompile (projectItem.getChild (i), shouldCompile);
     }
 
-    void showPopupMenu() override
+    void showPopupMenu (Point<int> p) override
     {
         PopupMenu m;
         addCreateFileMenuItems (m);
@@ -769,15 +810,15 @@ public:
             m.addItem (10, "Delete");
         }
 
-        launchPopupMenu (m);
+        launchPopupMenu (m, p);
     }
 
-    void showAddMenu() override
+    void showAddMenu (Point<int> p) override
     {
         PopupMenu m;
         addCreateFileMenuItems (m);
 
-        launchPopupMenu (m);
+        launchPopupMenu (m, p);
     }
 
     void handlePopupMenuResult (int resultCode) override
@@ -804,7 +845,7 @@ public:
         m.addItem (1002, "Add Existing Files...");
 
         m.addSeparator();
-        NewFileWizard().addWizardsToMenu (m);
+        wizard.addWizardsToMenu (m);
     }
 
     void processCreateFileMenuItem (int menuID)
@@ -816,7 +857,7 @@ public:
 
             default:
                 jassert (getProject() != nullptr);
-                NewFileWizard().runWizardFromMenu (menuID, *getProject(), item);
+                wizard.runWizardFromMenu (menuID, *getProject(), item);
                 break;
         }
     }
@@ -837,4 +878,5 @@ public:
     }
 
     String searchFilter;
+    NewFileWizard wizard;
 };
