@@ -2,17 +2,16 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2017 - ROLI Ltd.
+   Copyright (c) 2020 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
 
-   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
-   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
-   27th April 2017).
+   By using JUCE, you agree to the terms of both the JUCE 6 End-User License
+   Agreement and JUCE Privacy Policy (both effective as of the 16th June 2020).
 
-   End User License Agreement: www.juce.com/juce-5-licence
-   Privacy Policy: www.juce.com/juce-5-privacy-policy
+   End User License Agreement: www.juce.com/juce-6-licence
+   Privacy Policy: www.juce.com/juce-privacy-policy
 
    Or: You may also use this code under the terms of the GPL v3 (see
    www.gnu.org/licenses).
@@ -37,9 +36,10 @@ public:
         : project (p),
           modulesValueTree (project.getEnabledModules().getState())
     {
-        listHeader = new ListBoxHeader ( { "Module", "Version", "Make Local Copy", "Paths" },
-                                        { 0.25f, 0.2f, 0.2f, 0.35f } );
-        list.setHeaderComponent (listHeader);
+        auto tempHeader = std::make_unique<ListBoxHeader> (Array<String> { "Module", "Version", "Make Local Copy", "Paths" },
+                                                           Array<float> { 0.25f, 0.2f, 0.2f, 0.35f });
+        listHeader = tempHeader.get();
+        list.setHeaderComponent (std::move (tempHeader));
         list.setModel (this);
         list.setColour (ListBox::backgroundColourId, Colours::transparentBlack);
         addAndMakeVisible (list);
@@ -125,18 +125,18 @@ public:
         //==============================================================================
         auto moduleID = project.getEnabledModules().getModuleID (rowNumber);
 
-        g.drawFittedText (moduleID, bounds.removeFromLeft (roundToInt (listHeader->getProportionAtIndex (0) * width)), Justification::centredLeft, 1);
+        g.drawFittedText (moduleID, bounds.removeFromLeft (roundToInt (listHeader->getProportionAtIndex (0) * (float) width)), Justification::centredLeft, 1);
 
         //==============================================================================
         auto version = project.getEnabledModules().getModuleInfo (moduleID).getVersion();
         if (version.isEmpty())
             version = "?";
 
-        g.drawFittedText (version, bounds.removeFromLeft (roundToInt (listHeader->getProportionAtIndex (1) * width)), Justification::centredLeft, 1);
+        g.drawFittedText (version, bounds.removeFromLeft (roundToInt (listHeader->getProportionAtIndex (1) * (float) width)), Justification::centredLeft, 1);
 
         //==============================================================================
         g.drawFittedText (String (project.getEnabledModules().shouldCopyModuleFilesLocally (moduleID) ? "Yes" : "No"),
-                          bounds.removeFromLeft (roundToInt (listHeader->getProportionAtIndex (2) * width)), Justification::centredLeft, 1);
+                          bounds.removeFromLeft (roundToInt (listHeader->getProportionAtIndex (2) * (float) width)), Justification::centredLeft, 1);
 
         //==============================================================================
         String pathText;
@@ -152,10 +152,13 @@ public:
             for (Project::ExporterIterator exporter (project); exporter.next();)
                 paths.addIfNotAlreadyThere (exporter->getPathForModuleString (moduleID).trim());
 
+            paths.removeEmptyStrings();
+            paths.removeDuplicates (true);
+
             pathText = paths.joinIntoString (", ");
         }
 
-        g.drawFittedText (pathText, bounds.removeFromLeft (roundToInt (listHeader->getProportionAtIndex (3) * width)), Justification::centredLeft, 1);
+        g.drawFittedText (pathText, bounds.removeFromLeft (roundToInt (listHeader->getProportionAtIndex (3) * (float) width)), Justification::centredLeft, 1);
     }
 
     void listBoxItemDoubleClicked (int row, const MouseEvent&) override
@@ -282,13 +285,13 @@ private:
             m.addItem (PopupMenu::Item ("Copy the paths from the module '" + moduleToCopy + "' to all other modules")
                          .setAction ([this, moduleToCopy]
                                      {
-                                         auto& moduleList = project.getEnabledModules();
+                                         auto& modulesList = project.getEnabledModules();
 
                                          for (Project::ExporterIterator exporter (project); exporter.next();)
                                          {
-                                             for (int i = 0; i < moduleList.getNumModules(); ++i)
+                                             for (int i = 0; i < modulesList.getNumModules(); ++i)
                                              {
-                                                 auto modID = moduleList.getModuleID (i);
+                                                 auto modID = modulesList.getModuleID (i);
 
                                                  if (modID != moduleToCopy)
                                                      exporter->getPathForModuleValue (modID) = exporter->getPathForModuleValue (moduleToCopy).get();
@@ -305,7 +308,7 @@ private:
                                          modulePathClipboard.clear();
 
                                          for (Project::ExporterIterator exporter (project); exporter.next();)
-                                             modulePathClipboard[exporter->getName()] = exporter->getPathForModuleValue (moduleToCopy).get();
+                                             modulePathClipboard[exporter->getUniqueName()] = exporter->getPathForModuleValue (moduleToCopy).get();
 
                                          list.repaint();
                                      }));
@@ -320,7 +323,7 @@ private:
                                              auto modID = project.getEnabledModules().getModuleID (rowNumber);
 
                                              for (Project::ExporterIterator exporter (project); exporter.next();)
-                                                 exporter->getPathForModuleValue (modID) = modulePathClipboard[exporter->getName()];
+                                                 exporter->getPathForModuleValue (modID) = modulePathClipboard[exporter->getUniqueName()];
                                          }
 
                                          list.repaint();
