@@ -504,9 +504,6 @@ void ProjectContentComponent::openInSelectedIDE (bool saveFirst)
 
     if (auto selectedExporter = headerComponent.getSelectedExporter())
     {
-        if (! selectedExporter->canLaunchProject())
-            return;
-
         if (saveFirst)
         {
             if (project->isTemporaryProject())
@@ -515,15 +512,13 @@ void ProjectContentComponent::openInSelectedIDE (bool saveFirst)
                 return;
             }
 
-            if (project->hasChangedSinceSaved() || ! selectedExporter->getIDEProjectFile().exists())
-            {
-                project->saveAsync (true, true, [safeThis = SafePointer<ProjectContentComponent> { this }] (Project::SaveResult r)
-                {
-                    if (safeThis != nullptr && r == Project::SaveResult::savedOk)
-                        safeThis->openInSelectedIDE (false);
-                });
-                return;
-            }
+            SafePointer<ProjectContentComponent> safeThis { this };
+            project->saveAsync (true, true, [safeThis] (Project::SaveResult r)
+                                {
+                                    if (safeThis != nullptr && r == Project::SaveResult::savedOk)
+                                        safeThis->openInSelectedIDE (false);
+                                });
+            return;
         }
 
         project->openProjectInIDE (*selectedExporter);
