@@ -91,13 +91,17 @@ struct PropertyListBuilder
         add (propertyComp);
     }
 
-    void addSearchPathProperty (const Value& value, const String& name, const String& mainHelpText)
+    void addSearchPathProperty (const Value& value,
+                                const String& name,
+                                const String& mainHelpText)
     {
         add (new TextPropertyComponent (value, name, 16384, true),
              mainHelpText + " Use semi-colons or new-lines to separate multiple paths.");
     }
 
-    void addSearchPathProperty (ValueWithDefault& value, const String& name, const String& mainHelpText)
+    void addSearchPathProperty (const ValueTreePropertyWithDefault& value,
+                                const String& name,
+                                const String& mainHelpText)
     {
         add (new TextPropertyComponent (value, name, 16384, true),
              mainHelpText + " Use semi-colons or new-lines to separate multiple paths.");
