@@ -2,7 +2,7 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2017 - ROLI Ltd.
+   Copyright (c) 2020 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
@@ -29,37 +29,39 @@
 namespace juce
 {
 
+class MidiInput::Pimpl
+{
+public:
+    virtual ~Pimpl() noexcept = default;
+
+    virtual String getDeviceIdentifier() = 0;
+    virtual String getDeviceName() = 0;
+
+    virtual void start() = 0;
+    virtual void stop() = 0;
+};
+
+class MidiOutput::Pimpl
+{
+public:
+    virtual ~Pimpl() noexcept = default;
+
+    virtual String getDeviceIdentifier() = 0;
+    virtual String getDeviceName() = 0;
+
+    virtual void sendMessageNow (const MidiMessage&) = 0;
+};
+
 struct MidiServiceType
 {
-    struct InputWrapper
-    {
-        virtual ~InputWrapper() {}
-
-        virtual String getDeviceIdentifier() = 0;
-        virtual String getDeviceName() = 0;
-
-        virtual void start() = 0;
-        virtual void stop() = 0;
-    };
-
-    struct OutputWrapper
-    {
-        virtual ~OutputWrapper() {}
-
-        virtual String getDeviceIdentifier() = 0;
-        virtual String getDeviceName() = 0;
-
-        virtual void sendMessageNow (const MidiMessage&) = 0;
-    };
-
-    MidiServiceType() {}
-    virtual ~MidiServiceType() {}
+    MidiServiceType() = default;
+    virtual ~MidiServiceType() noexcept = default;
 
     virtual Array<MidiDeviceInfo> getAvailableDevices (bool) = 0;
     virtual MidiDeviceInfo getDefaultDevice (bool) = 0;
 
-    virtual InputWrapper*  createInputWrapper  (MidiInput&, const String&, MidiInputCallback&) = 0;
-    virtual OutputWrapper* createOutputWrapper (const String&) = 0;
+    virtual MidiInput::Pimpl*  createInputWrapper  (MidiInput&, const String&, MidiInputCallback&) = 0;
+    virtual MidiOutput::Pimpl* createOutputWrapper (const String&) = 0;
 
     JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (MidiServiceType)
 };
@@ -82,12 +84,12 @@ struct Win32MidiService  : public MidiServiceType,
                        : Win32OutputWrapper::getDefaultDevice();
     }
 
-    InputWrapper* createInputWrapper (MidiInput& input, const String& deviceIdentifier, MidiInputCallback& callback) override
+    MidiInput::Pimpl* createInputWrapper (MidiInput& input, const String& deviceIdentifier, MidiInputCallback& callback) override
     {
         return new Win32InputWrapper (*this, input, deviceIdentifier, callback);
     }
 
-    OutputWrapper* createOutputWrapper (const String& deviceIdentifier) override
+    MidiOutput::Pimpl* createOutputWrapper (const String& deviceIdentifier) override
     {
         return new Win32OutputWrapper (*this, deviceIdentifier);
     }
@@ -107,7 +109,7 @@ private:
         {
             stop();
 
-            if (deviceHandle != 0)
+            if (deviceHandle != nullptr)
             {
                 for (int count = 5; --count >= 0;)
                 {
@@ -181,7 +183,7 @@ private:
 
         void start()
         {
-            if (deviceHandle != 0 && ! isStarted.load())
+            if (deviceHandle != nullptr && ! isStarted.load())
             {
                 activeMidiCollectors.addIfNotAlreadyThere (this);
 
@@ -230,7 +232,7 @@ private:
         }
 
         MidiDeviceInfo deviceInfo;
-        HMIDIIN deviceHandle = 0;
+        HMIDIIN deviceHandle = nullptr;
 
     private:
         Win32MidiService& midiService;
@@ -256,7 +258,7 @@ private:
 
         struct MidiHeader
         {
-            MidiHeader() {}
+            MidiHeader() = default;
 
             void prepare (HMIDIIN device)
             {
@@ -332,7 +334,7 @@ private:
     };
 
     //==============================================================================
-    template<class WrapperType>
+    template <class WrapperType>
     struct Win32MidiDeviceQuery
     {
         static Array<MidiDeviceInfo> getAvailableDevices()
@@ -384,7 +386,7 @@ private:
         }
     };
 
-    struct Win32InputWrapper  : public InputWrapper,
+    struct Win32InputWrapper  : public MidiInput::Pimpl,
                                 public Win32MidiDeviceQuery<Win32InputWrapper>
     {
         Win32InputWrapper (Win32MidiService& parentService, MidiInput& midiInput, const String& deviceIdentifier, MidiInputCallback& c)
@@ -394,7 +396,7 @@ private:
             collector->addClient (this);
         }
 
-        ~Win32InputWrapper()
+        ~Win32InputWrapper() override
         {
             collector->removeClient (this);
         }
@@ -411,7 +413,7 @@ private:
 
                 if (d.identifier == deviceIdentifier)
                 {
-                    deviceID = i;
+                    deviceID = (UINT) i;
                     deviceName = d.name;
                     break;
                 }
@@ -450,7 +452,7 @@ private:
 
             for (UINT i = 0; i < midiInGetNumDevs(); ++i)
             {
-                MIDIINCAPS mc = { 0 };
+                MIDIINCAPS mc = {};
 
                 if (midiInGetDevCaps (i, &mc, sizeof (mc)) == MMSYSERR_NOERROR)
                     devices.add (mc);
@@ -508,7 +510,7 @@ private:
     };
 
     //==============================================================================
-    struct Win32OutputWrapper  : public OutputWrapper,
+    struct Win32OutputWrapper  : public MidiOutput::Pimpl,
                                  public Win32MidiDeviceQuery<Win32OutputWrapper>
     {
         Win32OutputWrapper (Win32MidiService& p, const String& deviceIdentifier)
@@ -524,7 +526,7 @@ private:
 
                 if (d.identifier == deviceIdentifier)
                 {
-                    deviceID = i;
+                    deviceID = (UINT) i;
                     deviceName = d.name;
                     break;
                 }
@@ -552,7 +554,7 @@ private:
 
             for (int i = 4; --i >= 0;)
             {
-                HMIDIOUT h = 0;
+                HMIDIOUT h = nullptr;
                 auto res = midiOutOpen (&h, deviceID, 0, 0, CALLBACK_NULL);
 
                 if (res == MMSYSERR_NOERROR)
@@ -574,7 +576,7 @@ private:
         {
             if (message.getRawDataSize() > 3 || message.isSysEx())
             {
-                MIDIHDR h = { 0 };
+                MIDIHDR h = {};
 
                 h.lpData = (char*) message.getRawData();
                 h.dwBytesRecorded = h.dwBufferLength  = (DWORD) message.getRawDataSize();
@@ -606,7 +608,7 @@ private:
             {
                 for (int i = 0; i < 50; ++i)
                 {
-                    if (midiOutShortMsg (han->handle, *(unsigned int*) message.getRawData()) != MIDIERR_NOTREADY)
+                    if (midiOutShortMsg (han->handle, *unalignedPointerCast<const unsigned int*> (message.getRawData())) != MIDIERR_NOTREADY)
                         break;
 
                     Sleep (1);
@@ -625,7 +627,7 @@ private:
 
             for (UINT i = 0; i < midiOutGetNumDevs(); ++i)
             {
-                MIDIOUTCAPS mc = { 0 };
+                MIDIOUTCAPS mc = {};
 
                 if (midiOutGetDevCaps (i, &mc, sizeof (mc)) == MMSYSERR_NOERROR)
                     devices.add (mc);
@@ -763,12 +765,12 @@ public:
                        : outputDeviceWatcher->getDefaultDevice();
     }
 
-    InputWrapper* createInputWrapper (MidiInput& input, const String& deviceIdentifier, MidiInputCallback& callback) override
+    MidiInput::Pimpl* createInputWrapper (MidiInput& input, const String& deviceIdentifier, MidiInputCallback& callback) override
     {
         return new WinRTInputWrapper (*this, input, deviceIdentifier, callback);
     }
 
-    OutputWrapper* createOutputWrapper (const String& deviceIdentifier) override
+    MidiOutput::Pimpl* createOutputWrapper (const String& deviceIdentifier) override
     {
         return new WinRTOutputWrapper (*this, deviceIdentifier);
     }
@@ -780,9 +782,9 @@ private:
     public:
         virtual ~DeviceCallbackHandler() {};
 
-        virtual HRESULT addDevice (IDeviceInformation*) = 0;
-        virtual HRESULT removeDevice (IDeviceInformationUpdate*) = 0;
-        virtual HRESULT updateDevice (IDeviceInformationUpdate*) = 0;
+        JUCE_COMCALL addDevice (IDeviceInformation*) = 0;
+        JUCE_COMCALL removeDevice (IDeviceInformationUpdate*) = 0;
+        JUCE_COMCALL updateDevice (IDeviceInformationUpdate*) = 0;
 
         bool attach (HSTRING deviceSelector, DeviceInformationKind infoKind)
         {
@@ -832,7 +834,7 @@ private:
                 }
             }
 
-            WinRTWrapper::ComPtr<IIterable<HSTRING>> iter;
+            ComSmartPtr<IIterable<HSTRING>> iter;
             auto hr = requestedProperties->QueryInterface (__uuidof (IIterable<HSTRING>), (void**) iter.resetAndGetPointerAddress());
 
             if (FAILED (hr))
@@ -889,7 +891,7 @@ private:
             watcher = nullptr;
         }
 
-        template<typename InfoType>
+        template <typename InfoType>
         IInspectable* getValueFromDeviceInfo (String key, InfoType* info)
         {
             __FIMapView_2_HSTRING_IInspectable* properties;
@@ -922,7 +924,7 @@ private:
 
         String getGUIDFromInspectable (IInspectable& inspectable)
         {
-            WinRTWrapper::ComPtr<IReference<GUID>> guidRef;
+            ComSmartPtr<IReference<GUID>> guidRef;
             auto hr = inspectable.QueryInterface (__uuidof (IReference<GUID>),
                                                   (void**) guidRef.resetAndGetPointerAddress());
 
@@ -949,7 +951,7 @@ private:
 
         bool getBoolFromInspectable (IInspectable& inspectable)
         {
-            WinRTWrapper::ComPtr<IReference<bool>> boolRef;
+            ComSmartPtr<IReference<bool>> boolRef;
             auto hr = inspectable.QueryInterface (__uuidof (IReference<bool>),
                                                   (void**) boolRef.resetAndGetPointerAddress());
 
@@ -976,7 +978,7 @@ private:
         struct DeviceEnumerationThread   : public Thread
         {
             DeviceEnumerationThread (DeviceCallbackHandler& h,
-                                     WinRTWrapper::ComPtr<IDeviceWatcher>& w,
+                                     ComSmartPtr<IDeviceWatcher>& w,
                                      EventRegistrationToken& added,
                                      EventRegistrationToken& removed,
                                      EventRegistrationToken& updated)
@@ -990,19 +992,19 @@ private:
 
                 watcher->add_Added (
                     Callback<ITypedEventHandler<DeviceWatcher*, DeviceInformation*>> (
-                        [handlerPtr](IDeviceWatcher*, IDeviceInformation* info) { return handlerPtr->addDevice (info); }
+                        [handlerPtr] (IDeviceWatcher*, IDeviceInformation* info) { return handlerPtr->addDevice (info); }
                     ).Get(),
                     &deviceAddedToken);
 
                 watcher->add_Removed (
                     Callback<ITypedEventHandler<DeviceWatcher*, DeviceInformationUpdate*>> (
-                        [handlerPtr](IDeviceWatcher*, IDeviceInformationUpdate* infoUpdate) { return handlerPtr->removeDevice (infoUpdate); }
+                        [handlerPtr] (IDeviceWatcher*, IDeviceInformationUpdate* infoUpdate) { return handlerPtr->removeDevice (infoUpdate); }
                     ).Get(),
                     &deviceRemovedToken);
 
                 watcher->add_Updated (
                     Callback<ITypedEventHandler<DeviceWatcher*, DeviceInformationUpdate*>> (
-                        [handlerPtr](IDeviceWatcher*, IDeviceInformationUpdate* infoUpdate) { return handlerPtr->updateDevice (infoUpdate); }
+                        [handlerPtr] (IDeviceWatcher*, IDeviceInformationUpdate* infoUpdate) { return handlerPtr->updateDevice (infoUpdate); }
                     ).Get(),
                     &deviceUpdatedToken);
 
@@ -1010,12 +1012,12 @@ private:
             }
 
             DeviceCallbackHandler& handler;
-            WinRTWrapper::ComPtr<IDeviceWatcher>& watcher;
+            ComSmartPtr<IDeviceWatcher>& watcher;
             EventRegistrationToken& deviceAddedToken, deviceRemovedToken, deviceUpdatedToken;
         };
 
         //==============================================================================
-        WinRTWrapper::ComPtr<IDeviceWatcher> watcher;
+        ComSmartPtr<IDeviceWatcher> watcher;
 
         EventRegistrationToken deviceAddedToken   { 0 },
                                deviceRemovedToken { 0 },
@@ -1118,7 +1120,7 @@ private:
                 if (devices.contains (removedDeviceId))
                 {
                     auto& info = devices.getReference (removedDeviceId);
-                    listeners.call ([&info](Listener& l) { l.bleDeviceDisconnected (info.containerID); });
+                    listeners.call ([&info] (Listener& l) { l.bleDeviceDisconnected (info.containerID); });
                     devices.remove (removedDeviceId);
                     JUCE_WINRT_MIDI_LOG ("Removed BLE device: " << removedDeviceId);
                 }
@@ -1165,7 +1167,7 @@ private:
                     if (info.isConnected && ! isConnected)
                     {
                         JUCE_WINRT_MIDI_LOG ("BLE device connection status change: " << updatedDeviceId << " " << info.containerID << " " << (isConnected ? "connected" : "disconnected"));
-                        listeners.call ([&info](Listener& l) { l.bleDeviceDisconnected (info.containerID); });
+                        listeners.call ([&info] (Listener& l) { l.bleDeviceDisconnected (info.containerID); });
                     }
 
                     info.isConnected = isConnected;
@@ -1220,7 +1222,7 @@ private:
     template <typename COMFactoryType>
     struct MidiIODeviceWatcher final   : private DeviceCallbackHandler
     {
-        MidiIODeviceWatcher (WinRTWrapper::ComPtr<COMFactoryType>& comFactory)
+        MidiIODeviceWatcher (ComSmartPtr<COMFactoryType>& comFactory)
             : factory (comFactory)
         {
         }
@@ -1405,7 +1407,7 @@ private:
             return {};
         }
 
-        WinRTWrapper::ComPtr<COMFactoryType>& factory;
+        ComSmartPtr<COMFactoryType>& factory;
 
         Array<WinRTMIDIDeviceInfo> connectedDevices;
         CriticalSection deviceChanges;
@@ -1419,8 +1421,8 @@ private:
     struct OpenMidiPortThread  : public Thread
     {
         OpenMidiPortThread (String threadName, String midiDeviceID,
-                            WinRTWrapper::ComPtr<COMFactoryType>& comFactory,
-                            WinRTWrapper::ComPtr<COMInterfaceType>& comPort)
+                            ComSmartPtr<COMFactoryType>& comFactory,
+                            ComSmartPtr<COMInterfaceType>& comPort)
             : Thread (threadName),
               deviceID (midiDeviceID),
               factory (comFactory),
@@ -1436,7 +1438,7 @@ private:
         void run() override
         {
             WinRTWrapper::ScopedHString hDeviceId (deviceID);
-            WinRTWrapper::ComPtr<IAsyncOperation<COMType*>> asyncOp;
+            ComSmartPtr<IAsyncOperation<COMType*>> asyncOp;
             auto hr = factory->FromIdAsync (hDeviceId.get(), asyncOp.resetAndGetPointerAddress());
 
             if (FAILED (hr))
@@ -1464,8 +1466,8 @@ private:
         }
 
         const String deviceID;
-        WinRTWrapper::ComPtr<COMFactoryType>& factory;
-        WinRTWrapper::ComPtr<COMInterfaceType>& port;
+        ComSmartPtr<COMFactoryType>& factory;
+        ComSmartPtr<COMInterfaceType>& port;
         WaitableEvent portOpened { true };
     };
 
@@ -1550,11 +1552,11 @@ private:
         BLEDeviceWatcher& bleDeviceWatcher;
         WinRTMIDIDeviceInfo deviceInfo;
         bool isBLEDevice = false;
-        WinRTWrapper::ComPtr<MIDIPort> midiPort;
+        ComSmartPtr<MIDIPort> midiPort;
     };
 
     //==============================================================================
-    struct WinRTInputWrapper final  : public InputWrapper,
+    struct WinRTInputWrapper final  : public MidiInput::Pimpl,
                                       private WinRTIOWrapper<IMidiInPortStatics, IMidiInPort>
 
     {
@@ -1580,7 +1582,7 @@ private:
 
             auto hr = midiPort->add_MessageReceived (
                 Callback<ITypedEventHandler<MidiInPort*, MidiMessageReceivedEventArgs*>> (
-                    [this](IMidiInPort*, IMidiMessageReceivedEventArgs* args) { return midiInMessageReceived (args); }
+                    [this] (IMidiInPort*, IMidiMessageReceivedEventArgs* args) { return midiInMessageReceived (args); }
                 ).Get(),
                 &midiInMessageToken);
 
@@ -1635,19 +1637,19 @@ private:
             if (! isStarted)
                 return S_OK;
 
-            WinRTWrapper::ComPtr<IMidiMessage> message;
+            ComSmartPtr<IMidiMessage> message;
             auto hr = args->get_Message (message.resetAndGetPointerAddress());
 
             if (FAILED (hr))
                 return hr;
 
-            WinRTWrapper::ComPtr<IBuffer> buffer;
+            ComSmartPtr<IBuffer> buffer;
             hr = message->get_RawData (buffer.resetAndGetPointerAddress());
 
             if (FAILED (hr))
                 return hr;
 
-            WinRTWrapper::ComPtr<Windows::Storage::Streams::IBufferByteAccess> bufferByteAccess;
+            ComSmartPtr<Windows::Storage::Streams::IBufferByteAccess> bufferByteAccess;
             hr = buffer->QueryInterface (bufferByteAccess.resetAndGetPointerAddress());
 
             if (FAILED (hr))
@@ -1708,7 +1710,7 @@ private:
     };
 
     //==============================================================================
-    struct WinRTOutputWrapper final  : public OutputWrapper,
+    struct WinRTOutputWrapper final  : public MidiOutput::Pimpl,
                                        private WinRTIOWrapper <IMidiOutPortStatics, IMidiOutPort>
     {
         WinRTOutputWrapper (WinRTMidiService& service, const String& deviceIdentifier)
@@ -1773,15 +1775,15 @@ private:
         String getDeviceName() override          { return deviceInfo.name; }
 
         //==============================================================================
-        WinRTWrapper::ComPtr<IBuffer> buffer;
-        WinRTWrapper::ComPtr<Windows::Storage::Streams::IBufferByteAccess> bufferByteAccess;
+        ComSmartPtr<IBuffer> buffer;
+        ComSmartPtr<Windows::Storage::Streams::IBufferByteAccess> bufferByteAccess;
         uint8_t* bufferData = nullptr;
 
         JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (WinRTOutputWrapper);
     };
 
-    WinRTWrapper::ComPtr<IMidiInPortStatics>  midiInFactory;
-    WinRTWrapper::ComPtr<IMidiOutPortStatics> midiOutFactory;
+    ComSmartPtr<IMidiInPortStatics>  midiInFactory;
+    ComSmartPtr<IMidiOutPortStatics> midiOutFactory;
 
     std::unique_ptr<MidiIODeviceWatcher<IMidiInPortStatics>>  inputDeviceWatcher;
     std::unique_ptr<MidiIODeviceWatcher<IMidiOutPortStatics>> outputDeviceWatcher;
@@ -1865,7 +1867,7 @@ std::unique_ptr<MidiInput> MidiInput::openDevice (const String& deviceIdentifier
         return {};
 
     std::unique_ptr<MidiInput> in (new MidiInput ({}, deviceIdentifier));
-    std::unique_ptr<MidiServiceType::InputWrapper> wrapper;
+    std::unique_ptr<Pimpl> wrapper;
 
     try
     {
@@ -1877,7 +1879,7 @@ std::unique_ptr<MidiInput> MidiInput::openDevice (const String& deviceIdentifier
     }
 
     in->setName (wrapper->getDeviceName());
-    in->internal = wrapper.release();
+    in->internal = std::move (wrapper);
 
     return in;
 }
@@ -1907,13 +1909,10 @@ MidiInput::MidiInput (const String& deviceName, const String& deviceIdentifier)
 {
 }
 
-MidiInput::~MidiInput()
-{
-    delete static_cast<MidiServiceType::InputWrapper*> (internal);
-}
+MidiInput::~MidiInput() = default;
 
-void MidiInput::start()   { static_cast<MidiServiceType::InputWrapper*> (internal)->start(); }
-void MidiInput::stop()    { static_cast<MidiServiceType::InputWrapper*> (internal)->stop(); }
+void MidiInput::start()   { internal->start(); }
+void MidiInput::stop()    { internal->stop(); }
 
 //==============================================================================
 Array<MidiDeviceInfo> MidiOutput::getAvailableDevices()
@@ -1931,7 +1930,7 @@ std::unique_ptr<MidiOutput> MidiOutput::openDevice (const String& deviceIdentifi
     if (deviceIdentifier.isEmpty())
         return {};
 
-    std::unique_ptr<MidiServiceType::OutputWrapper> wrapper;
+    std::unique_ptr<Pimpl> wrapper;
 
     try
     {
@@ -1945,7 +1944,7 @@ std::unique_ptr<MidiOutput> MidiOutput::openDevice (const String& deviceIdentifi
     std::unique_ptr<MidiOutput> out;
     out.reset (new MidiOutput (wrapper->getDeviceName(), deviceIdentifier));
 
-    out->internal = wrapper.release();
+    out->internal = std::move (wrapper);
 
     return out;
 }
@@ -1973,12 +1972,11 @@ std::unique_ptr<MidiOutput> MidiOutput::openDevice (int index)
 MidiOutput::~MidiOutput()
 {
     stopBackgroundThread();
-    delete static_cast<MidiServiceType::OutputWrapper*> (internal);
 }
 
 void MidiOutput::sendMessageNow (const MidiMessage& message)
 {
-    static_cast<MidiServiceType::OutputWrapper*> (internal)->sendMessageNow (message);
+    internal->sendMessageNow (message);
 }
 
 } // namespace juce
