@@ -2,17 +2,16 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2017 - ROLI Ltd.
+   Copyright (c) 2020 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
 
-   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
-   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
-   27th April 2017).
+   By using JUCE, you agree to the terms of both the JUCE 6 End-User License
+   Agreement and JUCE Privacy Policy (both effective as of the 16th June 2020).
 
-   End User License Agreement: www.juce.com/juce-5-licence
-   Privacy Policy: www.juce.com/juce-5-privacy-policy
+   End User License Agreement: www.juce.com/juce-6-licence
+   Privacy Policy: www.juce.com/juce-privacy-policy
 
    Or: You may also use this code under the terms of the GPL v3 (see
    www.gnu.org/licenses).
@@ -24,7 +23,8 @@
   ==============================================================================
 */
 
-#include "../../juce_core/system/juce_TargetPlatform.h"
+#include <juce_core/system/juce_CompilerWarnings.h>
+#include <juce_core/system/juce_TargetPlatform.h>
 #include "../utility/juce_CheckSettingMacros.h"
 
 #if JucePlugin_Build_RTAS
@@ -45,12 +45,9 @@
  #include <Mac2Win.H>
 #endif
 
-#ifdef __clang__
- #pragma clang diagnostic push
- #pragma clang diagnostic ignored "-Widiomatic-parentheses"
- #pragma clang diagnostic ignored "-Wnon-virtual-dtor"
- #pragma clang diagnostic ignored "-Wcomment"
-#endif
+JUCE_BEGIN_IGNORE_WARNINGS_GCC_LIKE ("-Widiomatic-parentheses",
+                                     "-Wnon-virtual-dtor",
+                                     "-Wcomment")
 
 /* Note about include paths
    ------------------------
@@ -99,15 +96,10 @@
 #include <FicProcessTokens.h>
 #include <ExternalVersionDefines.h>
 
-#ifdef __clang__
- #pragma clang diagnostic pop
-#endif
+JUCE_END_IGNORE_WARNINGS_GCC_LIKE
 
 //==============================================================================
-#ifdef _MSC_VER
- #pragma pack (push, 8)
- #pragma warning (disable: 4263 4264 4250)
-#endif
+JUCE_BEGIN_IGNORE_WARNINGS_MSVC (4263 4264 4250)
 
 #include "../utility/juce_IncludeModuleHeaders.h"
 
@@ -171,15 +163,15 @@ class JucePlugInProcess  : public CEffectProcessMIDI,
 {
 public:
     //==============================================================================
-    // RTAS builds will be removed from JUCE in the next release
-    JUCE_DEPRECATED_WITH_BODY (JucePlugInProcess(),
+    [[deprecated ("RTAS builds will be removed from JUCE in the next release.")]]
+    JucePlugInProcess()
     {
         juceFilter.reset (createPluginFilterOfType (AudioProcessor::wrapperType_RTAS));
 
         AddChunk (juceChunkType, "Juce Audio Plugin Data");
 
         ++numInstances;
-    })
+    }
 
     ~JucePlugInProcess()
     {
@@ -606,13 +598,9 @@ public:
         if (! midiEvents.isEmpty())
         {
            #if JucePlugin_ProducesMidiOutput
-            const juce::uint8* midiEventData;
-            int midiEventSize, midiEventPosition;
-            MidiBuffer::Iterator i (midiEvents);
-
-            while (i.getNextEvent (midiEventData, midiEventSize, midiEventPosition))
+            for (const auto metadata : midiEvents)
             {
-                //jassert (midiEventPosition >= 0 && midiEventPosition < (int) numSamples);
+                //jassert (metadata.samplePosition >= 0 && metadata.samplePosition < (int) numSamples);
             }
            #elif JUCE_DEBUG || JUCE_LOG_ASSERTIONS
             // if your plugin creates midi messages, you'll need to set
@@ -769,21 +757,24 @@ public:
         info.ppqLoopStart = 0;
         info.ppqLoopEnd = 0;
 
-        double framesPerSec = 24.0;
-
-        switch (fTimeCodeInfo.mFrameRate)
+        info.frameRate = [this]
         {
-            case ficFrameRate_24Frame:       info.frameRate = AudioPlayHead::fps24;       break;
-            case ficFrameRate_25Frame:       info.frameRate = AudioPlayHead::fps25;       framesPerSec = 25.0; break;
-            case ficFrameRate_2997NonDrop:   info.frameRate = AudioPlayHead::fps2997;     framesPerSec = 30.0 * 1000.0 / 1001.0; break;
-            case ficFrameRate_2997DropFrame: info.frameRate = AudioPlayHead::fps2997drop; framesPerSec = 30.0 * 1000.0 / 1001.0; break;
-            case ficFrameRate_30NonDrop:     info.frameRate = AudioPlayHead::fps30;       framesPerSec = 30.0; break;
-            case ficFrameRate_30DropFrame:   info.frameRate = AudioPlayHead::fps30drop;   framesPerSec = 30.0; break;
-            case ficFrameRate_23976:         info.frameRate = AudioPlayHead::fps23976;    framesPerSec = 24.0 * 1000.0 / 1001.0; break;
-            default:                         info.frameRate = AudioPlayHead::fpsUnknown;  break;
-        }
+            switch (fTimeCodeInfo.mFrameRate)
+            {
+                case ficFrameRate_24Frame:              return FrameRate().withBaseRate (24);
+                case ficFrameRate_23976:                return FrameRate().withBaseRate (24).withPullDown();
+                case ficFrameRate_25Frame:              return FrameRate().withBaseRate (25);
+                case ficFrameRate_30NonDrop:            return FrameRate().withBaseRate (30);
+                case ficFrameRate_30DropFrame:          return FrameRate().withBaseRate (30).withDrop();
+                case ficFrameRate_2997NonDrop:          return FrameRate().withBaseRate (30).withPullDown();
+                case ficFrameRate_2997DropFrame:        return FrameRate().withBaseRate (30).withPullDown().withDrop();
+            }
 
-        info.editOriginTime = fTimeCodeInfo.mFrameOffset / framesPerSec;
+            return FrameRate();
+        }();
+
+        const auto effectiveRate = info.frameRate.getEffectiveRate();
+        info.editOriginTime = effectiveRate != 0.0 ? fTimeCodeInfo.mFrameOffset / effectiveRate : 0.0;
 
         return true;
     }
@@ -803,7 +794,7 @@ public:
         ReleaseControl (index + 2);
     }
 
-    void audioProcessorChanged (AudioProcessor*) override
+    void audioProcessorChanged (AudioProcessor*, const ChangeDetails&) override
     {
         // xxx is there an RTAS equivalent?
     }
@@ -1061,4 +1052,6 @@ CProcessGroupInterface* CProcessGroup::CreateProcessGroup()
     return new JucePlugInGroup();
 }
 
+JUCE_END_IGNORE_WARNINGS_MSVC
+
 #endif
