@@ -2,17 +2,16 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2017 - ROLI Ltd.
+   Copyright (c) 2020 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
 
-   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
-   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
-   27th April 2017).
+   By using JUCE, you agree to the terms of both the JUCE 6 End-User License
+   Agreement and JUCE Privacy Policy (both effective as of the 16th June 2020).
 
-   End User License Agreement: www.juce.com/juce-5-licence
-   Privacy Policy: www.juce.com/juce-5-privacy-policy
+   End User License Agreement: www.juce.com/juce-6-licence
+   Privacy Policy: www.juce.com/juce-privacy-policy
 
    Or: You may also use this code under the terms of the GPL v3 (see
    www.gnu.org/licenses).
@@ -27,6 +26,14 @@
 namespace juce
 {
 
+static void updateOnMessageThread (AsyncUpdater& updater)
+{
+    if (MessageManager::getInstance()->isThisTheMessageThread())
+        updater.handleAsyncUpdate();
+    else
+        updater.triggerAsyncUpdate();
+}
+
 template <typename FloatType>
 struct GraphRenderSequence
 {
@@ -189,18 +196,19 @@ private:
     OwnedArray<RenderingOp> renderOps;
 
     //==============================================================================
-    template <typename LambdaType>
+    template <typename LambdaType,
+              std::enable_if_t<std::is_rvalue_reference<LambdaType&&>::value, int> = 0>
     void createOp (LambdaType&& fn)
     {
         struct LambdaOp  : public RenderingOp
         {
-            LambdaOp (LambdaType&& f) : function (std::move (f)) {}
+            LambdaOp (LambdaType&& f) : function (std::forward<LambdaType> (f)) {}
             void perform (const Context& c) override    { function (c); }
 
             LambdaType function;
         };
 
-        renderOps.add (new LambdaOp (std::move (fn)));
+        renderOps.add (new LambdaOp (std::forward<LambdaType> (fn)));
     }
 
     //==============================================================================
@@ -330,10 +338,8 @@ template <typename RenderSequence>
 struct RenderSequenceBuilder
 {
     RenderSequenceBuilder (AudioProcessorGraph& g, RenderSequence& s)
-        : graph (g), sequence (s)
+        : graph (g), sequence (s), orderedNodes (createOrderedNodeList (graph))
     {
-        createOrderedNodeList();
-
         audioBuffers.add (AssignedBuffer::createReadOnlyEmpty()); // first buffer is read-only zeros
         midiBuffers .add (AssignedBuffer::createReadOnlyEmpty());
 
@@ -351,12 +357,13 @@ struct RenderSequenceBuilder
     }
 
     //==============================================================================
+    using Node = AudioProcessorGraph::Node;
     using NodeID = AudioProcessorGraph::NodeID;
 
     AudioProcessorGraph& graph;
     RenderSequence& sequence;
 
-    Array<AudioProcessorGraph::Node*> orderedNodes;
+    const Array<Node*> orderedNodes;
 
     struct AssignedBuffer
     {
@@ -408,21 +415,58 @@ struct RenderSequenceBuilder
     }
 
     //==============================================================================
-    void createOrderedNodeList()
+    static void getAllParentsOfNode (const Node* child,
+                                     std::unordered_set<Node*>& parents,
+                                     const std::unordered_map<Node*, std::unordered_set<Node*>>& otherParents)
     {
-        for (auto* node : graph.getNodes())
+        for (auto&& i : child->inputs)
         {
-            int j = 0;
+            auto* parentNode = i.otherNode;
 
-            for (; j < orderedNodes.size(); ++j)
-                if (graph.isAnInputTo (*node, *orderedNodes.getUnchecked(j)))
-                  break;
+            if (parentNode == child)
+                continue;
 
-            orderedNodes.insert (j, node);
+            if (parents.insert (parentNode).second)
+            {
+                auto parentParents = otherParents.find (parentNode);
+
+                if (parentParents != otherParents.end())
+                {
+                    parents.insert (parentParents->second.begin(), parentParents->second.end());
+                    continue;
+                }
+
+                getAllParentsOfNode (i.otherNode, parents, otherParents);
+            }
         }
     }
 
-    int findBufferForInputAudioChannel (AudioProcessorGraph::Node& node, const int inputChan,
+    static auto createOrderedNodeList (const AudioProcessorGraph& graph)
+    {
+        Array<Node*> result;
+
+        std::unordered_map<Node*, std::unordered_set<Node*>> nodeParents;
+
+        for (auto* node : graph.getNodes())
+        {
+            int insertionIndex = 0;
+
+            for (; insertionIndex < result.size(); ++insertionIndex)
+            {
+                auto& parents = nodeParents[result.getUnchecked (insertionIndex)];
+
+                if (parents.find (node) != parents.end())
+                    break;
+            }
+
+            result.insert (insertionIndex, node);
+            getAllParentsOfNode (node, nodeParents[node], nodeParents);
+        }
+
+        return result;
+    }
+
+    int findBufferForInputAudioChannel (Node& node, const int inputChan,
                                         const int ourRenderingIndex, const int maxLatency)
     {
         auto& processor = *node.getProcessor();
@@ -554,7 +598,7 @@ struct RenderSequenceBuilder
         return bufIndex;
     }
 
-    int findBufferForInputMidiChannel (AudioProcessorGraph::Node& node, int ourRenderingIndex)
+    int findBufferForInputMidiChannel (Node& node, int ourRenderingIndex)
     {
         auto& processor = *node.getProcessor();
         auto sources = getSourcesForChannel (node, AudioProcessorGraph::midiChannelIndex);
@@ -645,7 +689,7 @@ struct RenderSequenceBuilder
         return midiBufferToUse;
     }
 
-    void createRenderingOpsForNode (AudioProcessorGraph::Node& node, const int ourRenderingIndex)
+    void createRenderingOpsForNode (Node& node, const int ourRenderingIndex)
     {
         auto& processor = *node.getProcessor();
         auto numIns  = processor.getTotalNumInputChannels();
@@ -690,7 +734,7 @@ struct RenderSequenceBuilder
     }
 
     //==============================================================================
-    Array<AudioProcessorGraph::NodeAndChannel> getSourcesForChannel (AudioProcessorGraph::Node& node, int inputChannelIndex)
+    Array<AudioProcessorGraph::NodeAndChannel> getSourcesForChannel (Node& node, int inputChannelIndex)
     {
         Array<AudioProcessorGraph::NodeAndChannel> results;
         AudioProcessorGraph::NodeAndChannel nc { node.nodeID, inputChannelIndex };
@@ -901,7 +945,7 @@ void AudioProcessorGraph::topologyChanged()
     sendChangeMessage();
 
     if (isPrepared)
-        triggerAsyncUpdate();
+        updateOnMessageThread (*this);
 }
 
 void AudioProcessorGraph::clear()
@@ -961,31 +1005,31 @@ AudioProcessorGraph::Node::Ptr AudioProcessorGraph::addNode (std::unique_ptr<Aud
     return n;
 }
 
-bool AudioProcessorGraph::removeNode (NodeID nodeId)
+AudioProcessorGraph::Node::Ptr AudioProcessorGraph::removeNode (NodeID nodeId)
 {
     const ScopedLock sl (getCallbackLock());
 
     for (int i = nodes.size(); --i >= 0;)
     {
-        if (nodes.getUnchecked(i)->nodeID == nodeId)
+        if (nodes.getUnchecked (i)->nodeID == nodeId)
         {
             disconnectNode (nodeId);
-            nodes.remove (i);
+            auto node = nodes.removeAndReturn (i);
             topologyChanged();
-            return true;
+            return node;
         }
     }
 
-    return false;
+    return {};
 }
 
-bool AudioProcessorGraph::removeNode (Node* node)
+AudioProcessorGraph::Node::Ptr AudioProcessorGraph::removeNode (Node* node)
 {
     if (node != nullptr)
         return removeNode (node->nodeID);
 
     jassertfalse;
-    return false;
+    return {};
 }
 
 //==============================================================================
@@ -1258,14 +1302,27 @@ void AudioProcessorGraph::prepareToPlay (double sampleRate, int estimatedSamples
     {
         const ScopedLock sl (getCallbackLock());
         setRateAndBufferSizeDetails (sampleRate, estimatedSamplesPerBlock);
+
+        const auto newPrepareSettings = [&]
+        {
+            PrepareSettings settings;
+            settings.precision  = getProcessingPrecision();
+            settings.sampleRate = sampleRate;
+            settings.blockSize  = estimatedSamplesPerBlock;
+            settings.valid      = true;
+            return settings;
+        }();
+
+        if (prepareSettings != newPrepareSettings)
+        {
+            unprepare();
+            prepareSettings = newPrepareSettings;
+        }
     }
 
     clearRenderingSequence();
 
-    if (MessageManager::getInstance()->isThisTheMessageThread())
-        handleAsyncUpdate();
-    else
-        triggerAsyncUpdate();
+    updateOnMessageThread (*this);
 }
 
 bool AudioProcessorGraph::supportsDoublePrecisionProcessing() const
@@ -1273,16 +1330,23 @@ bool AudioProcessorGraph::supportsDoublePrecisionProcessing() const
     return true;
 }
 
+void AudioProcessorGraph::unprepare()
+{
+    prepareSettings.valid = false;
+
+    isPrepared = 0;
+
+    for (auto* n : nodes)
+        n->unprepare();
+}
+
 void AudioProcessorGraph::releaseResources()
 {
     const ScopedLock sl (getCallbackLock());
 
     cancelPendingUpdate();
 
-    isPrepared = 0;
-
-    for (auto* n : nodes)
-        n->unprepare();
+    unprepare();
 
     if (renderSequenceFloat != nullptr)
         renderSequenceFloat->releaseBuffers();
@@ -1312,7 +1376,7 @@ void AudioProcessorGraph::setNonRealtime (bool isProcessingNonRealtime) noexcept
 double AudioProcessorGraph::getTailLengthSeconds() const            { return 0; }
 bool AudioProcessorGraph::acceptsMidi() const                       { return true; }
 bool AudioProcessorGraph::producesMidi() const                      { return true; }
-void AudioProcessorGraph::getStateInformation (juce::MemoryBlock&)  {}
+void AudioProcessorGraph::getStateInformation (MemoryBlock&)        {}
 void AudioProcessorGraph::setStateInformation (const void*, int)    {}
 
 template <typename FloatType, typename SequenceType>
@@ -1380,8 +1444,8 @@ const String AudioProcessorGraph::AudioGraphIOProcessor::getName() const
     {
         case audioOutputNode:   return "Audio Output";
         case audioInputNode:    return "Audio Input";
-        case midiOutputNode:    return "Midi Output";
-        case midiInputNode:     return "Midi Input";
+        case midiOutputNode:    return "MIDI Output";
+        case midiInputNode:     return "MIDI Input";
         default:                break;
     }
 
@@ -1391,13 +1455,14 @@ const String AudioProcessorGraph::AudioGraphIOProcessor::getName() const
 void AudioProcessorGraph::AudioGraphIOProcessor::fillInPluginDescription (PluginDescription& d) const
 {
     d.name = getName();
-    d.uid = d.name.hashCode();
     d.category = "I/O devices";
     d.pluginFormatName = "Internal";
     d.manufacturerName = "JUCE";
     d.version = "1.0";
     d.isInstrument = false;
 
+    d.deprecatedUid = d.uniqueId = d.name.hashCode();
+
     d.numInputChannels = getTotalNumInputChannels();
 
     if (type == audioOutputNode && graph != nullptr)
@@ -1502,7 +1567,7 @@ void AudioProcessorGraph::AudioGraphIOProcessor::setCurrentProgram (int)
 const String AudioProcessorGraph::AudioGraphIOProcessor::getProgramName (int)       { return {}; }
 void AudioProcessorGraph::AudioGraphIOProcessor::changeProgramName (int, const String&) {}
 
-void AudioProcessorGraph::AudioGraphIOProcessor::getStateInformation (juce::MemoryBlock&) {}
+void AudioProcessorGraph::AudioGraphIOProcessor::getStateInformation (MemoryBlock&)     {}
 void AudioProcessorGraph::AudioGraphIOProcessor::setStateInformation (const void*, int) {}
 
 void AudioProcessorGraph::AudioGraphIOProcessor::setParentGraph (AudioProcessorGraph* const newGraph)
