@@ -2,17 +2,16 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2017 - ROLI Ltd.
+   Copyright (c) 2020 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
 
-   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
-   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
-   27th April 2017).
+   By using JUCE, you agree to the terms of both the JUCE 6 End-User License
+   Agreement and JUCE Privacy Policy (both effective as of the 16th June 2020).
 
-   End User License Agreement: www.juce.com/juce-5-licence
-   Privacy Policy: www.juce.com/juce-5-privacy-policy
+   End User License Agreement: www.juce.com/juce-6-licence
+   Privacy Policy: www.juce.com/juce-privacy-policy
 
    Or: You may also use this code under the terms of the GPL v3 (see
    www.gnu.org/licenses).
@@ -74,7 +73,7 @@ private:
             parameterValueHasChanged = 1;
     }
 
-    void audioProcessorChanged (AudioProcessor*) override {}
+    void audioProcessorChanged (AudioProcessor*, const ChangeDetails&) override {}
 
     //==============================================================================
     void timerCallback() override
@@ -285,7 +284,7 @@ private:
         {
             // The parameter is producing some unexpected text, so we'll do
             // some linear interpolation.
-            index = roundToInt (getParameter().getValue() * (parameterValues.size() - 1));
+            index = roundToInt (getParameter().getValue() * (float) (parameterValues.size() - 1));
         }
 
         box.setSelectedItemIndex (index);
@@ -325,6 +324,8 @@ public:
         else
             slider.setRange (0.0, 1.0);
 
+        slider.setDoubleClickReturnValue (true, param.getDefaultValue());
+
         slider.setScrollWheelEnabled (false);
         addAndMakeVisible (slider);
 
@@ -405,25 +406,35 @@ private:
 };
 
 //==============================================================================
-class ParameterDisplayComponent   : public Component
+class ParameterDisplayComponent   : public Component,
+                                    private AudioProcessorListener,
+                                    private AsyncUpdater
 {
 public:
-    ParameterDisplayComponent (AudioProcessor& processor, AudioProcessorParameter& param)
-        : parameter (param)
+    ParameterDisplayComponent (AudioProcessorEditor& editorIn, AudioProcessorParameter& param)
+        : editor (editorIn), parameter (param)
     {
+        editor.processor.addListener (this);
+
         parameterName.setText (parameter.getName (128), dontSendNotification);
         parameterName.setJustificationType (Justification::centredRight);
+        parameterName.setInterceptsMouseClicks (false, false);
         addAndMakeVisible (parameterName);
 
         parameterLabel.setText (parameter.getLabel(), dontSendNotification);
+        parameterLabel.setInterceptsMouseClicks (false, false);
         addAndMakeVisible (parameterLabel);
 
-        addAndMakeVisible (*(parameterComp = createParameterComp (processor)));
+        addAndMakeVisible (*(parameterComp = createParameterComp (editor.processor)));
 
         setSize (400, 40);
     }
 
-    void paint (Graphics&) override {}
+    ~ParameterDisplayComponent() override
+    {
+        cancelPendingUpdate();
+        editor.processor.removeListener (this);
+    }
 
     void resized() override
     {
@@ -434,7 +445,17 @@ public:
         parameterComp->setBounds (area);
     }
 
+    void mouseDown (const MouseEvent& e) override
+    {
+        if (e.mods.isRightButtonDown())
+            if (auto* context = editor.getHostContext())
+                if (auto menu = context->getContextMenuForParameterIndex (&parameter))
+                    menu->getEquivalentPopupMenu().showMenuAsync (PopupMenu::Options().withTargetComponent (this)
+                                                                                      .withMousePosition());
+    }
+
 private:
+    AudioProcessorEditor& editor;
     AudioProcessorParameter& parameter;
     Label parameterName, parameterLabel;
     std::unique_ptr<Component> parameterComp;
@@ -463,103 +484,124 @@ private:
         return std::make_unique<SliderParameterComponent> (processor, parameter);
     }
 
+    void audioProcessorParameterChanged (AudioProcessor*, int, float) override {}
+
+    void audioProcessorChanged (AudioProcessor*, const ChangeDetails& details) override
+    {
+        if (! details.parameterInfoChanged)
+            return;
+
+        if (MessageManager::getInstance()->isThisTheMessageThread())
+            handleAsyncUpdate();
+        else
+            triggerAsyncUpdate();
+    }
+
+    void handleAsyncUpdate() override
+    {
+        parameterName .setText (parameter.getName (128), dontSendNotification);
+        parameterLabel.setText (parameter.getLabel(),    dontSendNotification);
+    }
+
     JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (ParameterDisplayComponent)
 };
 
 //==============================================================================
-class ParametersPanel   : public Component
+struct ParamControlItem : public TreeViewItem
 {
-public:
-    ParametersPanel (AudioProcessor& processor, const Array<AudioProcessorParameter*>& parameters)
+    ParamControlItem (AudioProcessorEditor& editorIn, AudioProcessorParameter& paramIn)
+        : editor (editorIn), param (paramIn) {}
+
+    bool mightContainSubItems() override { return false; }
+
+    std::unique_ptr<Component> createItemComponent() override
     {
-        for (auto* param : parameters)
-            if (param->isAutomatable())
-                addAndMakeVisible (paramComponents.add (new ParameterDisplayComponent (processor, *param)));
+        return std::make_unique<ParameterDisplayComponent> (editor, param);
+    }
 
-        int maxWidth = 400;
-        int height = 0;
+    int getItemHeight() const override { return 40; }
 
-        for (auto& comp : paramComponents)
+    AudioProcessorEditor& editor;
+    AudioProcessorParameter& param;
+};
+
+struct ParameterGroupItem : public TreeViewItem
+{
+    ParameterGroupItem (AudioProcessorEditor& editor, const AudioProcessorParameterGroup& group)
+        : name (group.getName())
+    {
+        for (auto* node : group)
         {
-            maxWidth = jmax (maxWidth, comp->getWidth());
-            height += comp->getHeight();
+            if (auto* param = node->getParameter())
+                if (param->isAutomatable())
+                    addSubItem (new ParamControlItem (editor, *param));
+
+            if (auto* inner = node->getGroup())
+            {
+                auto groupItem = std::make_unique<ParameterGroupItem> (editor, *inner);
+
+                if (groupItem->getNumSubItems() != 0)
+                    addSubItem (groupItem.release());
+            }
         }
-
-        setSize (maxWidth, jmax (height, 125));
     }
 
-    ~ParametersPanel() override
+    bool mightContainSubItems() override { return getNumSubItems() > 0; }
+
+    std::unique_ptr<Component> createItemComponent() override
     {
-        paramComponents.clear();
+        return std::make_unique<Label> (name, name);
     }
 
-    void paint (Graphics& g) override
-    {
-        g.fillAll (getLookAndFeel().findColour (ResizableWindow::backgroundColourId));
-    }
-
-    void resized() override
-    {
-        auto area = getLocalBounds();
-
-        for (auto* comp : paramComponents)
-            comp->setBounds (area.removeFromTop (comp->getHeight()));
-    }
-
-private:
-    OwnedArray<ParameterDisplayComponent> paramComponents;
-
-    JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (ParametersPanel)
+    String name;
 };
 
 //==============================================================================
 struct GenericAudioProcessorEditor::Pimpl
 {
-    Pimpl (GenericAudioProcessorEditor& parent)  : owner (parent)
+    Pimpl (AudioProcessorEditor& editor)
+        : legacyParameters (editor.processor, false),
+          groupItem (editor, legacyParameters.getGroup())
     {
-        auto* p = parent.getAudioProcessor();
-        jassert (p != nullptr);
+        const auto numIndents = getNumIndents (groupItem);
+        const auto width = 400 + view.getIndentSize() * numIndents;
 
-        legacyParameters.update (*p, false);
-
-        owner.setOpaque (true);
-
-        view.setViewedComponent (new ParametersPanel (*p, legacyParameters.params));
-        owner.addAndMakeVisible (view);
-
-        view.setScrollBarsShown (true, false);
+        view.setSize (width, 400);
+        view.setDefaultOpenness (true);
+        view.setRootItemVisible (false);
+        view.setRootItem (&groupItem);
     }
 
-    ~Pimpl()
+    static int getNumIndents (const TreeViewItem& item)
     {
-        view.setViewedComponent (nullptr, false);
+        int maxInner = 0;
+
+        for (auto i = 0; i < item.getNumSubItems(); ++i)
+            maxInner = jmax (maxInner, 1 + getNumIndents (*item.getSubItem (i)));
+
+        return maxInner;
     }
 
-    void resize (Rectangle<int> size)
-    {
-        view.setBounds (size);
-        auto content = view.getViewedComponent();
-        content->setSize (view.getMaximumVisibleWidth(), content->getHeight());
-    }
-
-    //==============================================================================
-    GenericAudioProcessorEditor& owner;
     LegacyAudioParametersWrapper legacyParameters;
-    Viewport view;
-
-    JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (Pimpl)
+    ParameterGroupItem groupItem;
+    TreeView view;
 };
 
-
 //==============================================================================
 GenericAudioProcessorEditor::GenericAudioProcessorEditor (AudioProcessor& p)
-    : AudioProcessorEditor (p), pimpl (new Pimpl (*this))
+    : AudioProcessorEditor (p), pimpl (std::make_unique<Pimpl> (*this))
 {
-    setSize (pimpl->view.getViewedComponent()->getWidth() + pimpl->view.getVerticalScrollBar().getWidth(),
-             jmin (pimpl->view.getViewedComponent()->getHeight(), 400));
+    auto* viewport = pimpl->view.getViewport();
+
+    setOpaque (true);
+    addAndMakeVisible (pimpl->view);
+
+    setResizable (true, false);
+    setSize (viewport->getViewedComponent()->getWidth() + viewport->getVerticalScrollBar().getWidth(),
+             jlimit (125, 400, viewport->getViewedComponent()->getHeight()));
 }
 
-GenericAudioProcessorEditor::~GenericAudioProcessorEditor() {}
+GenericAudioProcessorEditor::~GenericAudioProcessorEditor() = default;
 
 void GenericAudioProcessorEditor::paint (Graphics& g)
 {
@@ -568,7 +610,7 @@ void GenericAudioProcessorEditor::paint (Graphics& g)
 
 void GenericAudioProcessorEditor::resized()
 {
-    pimpl->resize (getLocalBounds());
+    pimpl->view.setBounds (getLocalBounds());
 }
 
 } // namespace juce
