@@ -2,7 +2,7 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2017 - ROLI Ltd.
+   Copyright (c) 2020 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
@@ -843,9 +843,9 @@ public:
 
     //==============================================================================
    #ifndef DOXYGEN
-    // Note that the swapWithArray method has been replaced by a more flexible templated version,
-    // and renamed "swapWith" to be more consistent with the names used in other classes.
-    JUCE_DEPRECATED_WITH_BODY (void swapWithArray (OwnedArray& other) noexcept, { swapWith (other); })
+    [[deprecated ("This method has been replaced by a more flexible templated version and renamed "
+                 "to swapWith to be more consistent with the names used in other classes.")]]
+    void swapWithArray (OwnedArray& other) noexcept { swapWith (other); }
    #endif
 
 private:
