@@ -969,6 +969,7 @@ public:
 
         if (! connection->start (owner, webInputListener))
         {
+            const auto errorCode = connection->getErrorCode();
             connection.reset();
 
             if (@available (macOS 10.10, *))
@@ -976,7 +977,7 @@ public:
 
             // Workaround for macOS versions below 10.10 where HTTPS POST requests with keep-alive
             // fail with the NSURLErrorNetworkConnectionLost error code.
-            if (numRetries == 0 && connection->getErrorCode() == NSURLErrorNetworkConnectionLost)
+            if (numRetries == 0 && errorCode == NSURLErrorNetworkConnectionLost)
                 return connect (webInputListener, ++numRetries);
 
             return false;
