@@ -37,6 +37,12 @@ namespace SampleTypeHelpers // Internal classes needed for handling sample type
         using Type = T;
     };
 
+    template <typename T>
+    struct ElementType<const T, false>
+    {
+        using Type = const typename T::value_type;
+    };
+
     template <typename T>
     struct ElementType<T, false>
     {
