@@ -2,17 +2,16 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2017 - ROLI Ltd.
+   Copyright (c) 2020 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
 
-   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
-   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
-   27th April 2017).
+   By using JUCE, you agree to the terms of both the JUCE 6 End-User License
+   Agreement and JUCE Privacy Policy (both effective as of the 16th June 2020).
 
-   End User License Agreement: www.juce.com/juce-5-licence
-   Privacy Policy: www.juce.com/juce-5-privacy-policy
+   End User License Agreement: www.juce.com/juce-6-licence
+   Privacy Policy: www.juce.com/juce-privacy-policy
 
    Or: You may also use this code under the terms of the GPL v3 (see
    www.gnu.org/licenses).
@@ -414,10 +413,12 @@ public:
     /** @internal */
     explicit Image (ReferenceCountedObjectPtr<ImagePixelData>) noexcept;
 
-    /* A null Image object that can be used when you need to return an invalid image.
-        @deprecated If you need a default-constructed var, just use Image() or {}.
-    */
-    JUCE_DEPRECATED_STATIC (static const Image null;)
+    //==============================================================================
+   #if JUCE_ALLOW_STATIC_NULL_VARIABLES && ! defined (DOXYGEN)
+    /* A null Image object that can be used when you need to return an invalid image. */
+    [[deprecated ("If you need a default-constructed var, just use Image() or {}.")]]
+    static const Image null;
+   #endif
 
 private:
     //==============================================================================
