@@ -2,17 +2,16 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2017 - ROLI Ltd.
+   Copyright (c) 2020 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
 
-   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
-   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
-   27th April 2017).
+   By using JUCE, you agree to the terms of both the JUCE 6 End-User License
+   Agreement and JUCE Privacy Policy (both effective as of the 16th June 2020).
 
-   End User License Agreement: www.juce.com/juce-5-licence
-   Privacy Policy: www.juce.com/juce-5-privacy-policy
+   End User License Agreement: www.juce.com/juce-6-licence
+   Privacy Policy: www.juce.com/juce-privacy-policy
 
    Or: You may also use this code under the terms of the GPL v3 (see
    www.gnu.org/licenses).
@@ -113,11 +112,6 @@ FileBrowserComponent::FileBrowserComponent (int flags_,
     addAndMakeVisible (fileLabel);
     fileLabel.attachToComponent (&filenameBox, true);
 
-    goUpButton.reset (getLookAndFeel().createFileBrowserGoUpButton());
-    addAndMakeVisible (goUpButton.get());
-    goUpButton->onClick = [this] { goUp(); };
-    goUpButton->setTooltip (TRANS ("Go up to parent directory"));
-
     if (previewComp != nullptr)
         addAndMakeVisible (previewComp);
 
@@ -359,12 +353,24 @@ void FileBrowserComponent::resized()
 //==============================================================================
 void FileBrowserComponent::lookAndFeelChanged()
 {
+    goUpButton.reset (getLookAndFeel().createFileBrowserGoUpButton());
+
+    if (auto* buttonPtr = goUpButton.get())
+    {
+        addAndMakeVisible (*buttonPtr);
+        buttonPtr->onClick = [this] { goUp(); };
+        buttonPtr->setTooltip (TRANS ("Go up to parent directory"));
+    }
+
     currentPathBox.setColour (ComboBox::backgroundColourId,    findColour (currentPathBoxBackgroundColourId));
     currentPathBox.setColour (ComboBox::textColourId,          findColour (currentPathBoxTextColourId));
     currentPathBox.setColour (ComboBox::arrowColourId,         findColour (currentPathBoxArrowColourId));
 
     filenameBox.setColour (TextEditor::backgroundColourId,     findColour (filenameBoxBackgroundColourId));
-    filenameBox.setColour (TextEditor::textColourId,           findColour (filenameBoxTextColourId));
+    filenameBox.applyColourToAllText (findColour (filenameBoxTextColourId));
+
+    resized();
+    repaint();
 }
 
 //==============================================================================
@@ -435,7 +441,7 @@ void FileBrowserComponent::browserRootChanged (const File&) {}
 
 bool FileBrowserComponent::keyPressed (const KeyPress& key)
 {
-   #if JUCE_LINUX || JUCE_WINDOWS
+   #if JUCE_LINUX || JUCE_BSD || JUCE_WINDOWS
     if (key.getModifiers().isCommandDown()
          && (key.getKeyCode() == 'H' || key.getKeyCode() == 'h'))
     {
@@ -599,7 +605,7 @@ void FileBrowserComponent::getRoots (StringArray& rootNames, StringArray& rootPa
 
 void FileBrowserComponent::timerCallback()
 {
-    const bool isProcessActive = Process::isForegroundProcess();
+    const auto isProcessActive = isForegroundOrEmbeddedProcess (this);
 
     if (wasProcessActive != isProcessActive)
     {
@@ -610,4 +616,10 @@ void FileBrowserComponent::timerCallback()
     }
 }
 
+//==============================================================================
+std::unique_ptr<AccessibilityHandler> FileBrowserComponent::createAccessibilityHandler()
+{
+    return std::make_unique<AccessibilityHandler> (*this, AccessibilityRole::group);
+}
+
 } // namespace juce
