@@ -2,17 +2,16 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2017 - ROLI Ltd.
+   Copyright (c) 2020 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
 
-   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
-   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
-   27th April 2017).
+   By using JUCE, you agree to the terms of both the JUCE 6 End-User License
+   Agreement and JUCE Privacy Policy (both effective as of the 16th June 2020).
 
-   End User License Agreement: www.juce.com/juce-5-licence
-   Privacy Policy: www.juce.com/juce-5-privacy-policy
+   End User License Agreement: www.juce.com/juce-6-licence
+   Privacy Policy: www.juce.com/juce-privacy-policy
 
    Or: You may also use this code under the terms of the GPL v3 (see
    www.gnu.org/licenses).
@@ -314,10 +313,13 @@ public:
     };
 
     //==============================================================================
-    // Deprecated: use setContentOwned() and setContentNonOwned() instead.
-    JUCE_DEPRECATED (void setContentComponent (Component* newContentComponent,
-                                               bool deleteOldOne = true,
-                                               bool resizeToFit = false));
+   #ifndef DOXYGEN
+    [[deprecated ("use setContentOwned and setContentNonOwned instead.")]]
+    void setContentComponent (Component* newContentComponent,
+                              bool deleteOldOne = true,
+                              bool resizeToFit = false);
+   #endif
+
     using TopLevelWindow::addToDesktop;
 
     //==============================================================================
@@ -382,6 +384,11 @@ protected:
     std::unique_ptr<ResizableCornerComponent> resizableCorner;
     std::unique_ptr<ResizableBorderComponent> resizableBorder;
 
+    //==============================================================================
+    // The parameters for these methods have changed - please update your code!
+    void getBorderThickness (int& left, int& top, int& right, int& bottom);
+    void getContentComponentBorder (int& left, int& top, int& right, int& bottom);
+
 private:
     //==============================================================================
     Component::SafePointer<Component> contentComponent, splashScreen;
@@ -400,12 +407,6 @@ private:
     void setContent (Component*, bool takeOwnership, bool resizeToFit);
     void updatePeerConstrainer();
 
-   #if JUCE_CATCH_DEPRECATED_CODE_MISUSE
-    // The parameters for these methods have changed - please update your code!
-    JUCE_DEPRECATED (void getBorderThickness (int& left, int& top, int& right, int& bottom));
-    JUCE_DEPRECATED (void getContentComponentBorder (int& left, int& top, int& right, int& bottom));
-   #endif
-
     JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (ResizableWindow)
 };
 
