@@ -71,13 +71,13 @@ void TooltipWindow::mouseEnter (const MouseEvent& e)
 void TooltipWindow::mouseDown (const MouseEvent&)
 {
     if (isVisible())
-        dismissalMouseEventOccured = true;
+        dismissalMouseEventOccurred = true;
 }
 
 void TooltipWindow::mouseWheelMove (const MouseEvent&, const MouseWheelDetails&)
 {
     if (isVisible())
-        dismissalMouseEventOccured = true;
+        dismissalMouseEventOccurred = true;
 }
 
 void TooltipWindow::updatePosition (const String& tip, Point<int> pos, Rectangle<int> parentArea)
@@ -145,7 +145,7 @@ void TooltipWindow::displayTipInternal (Point<int> screenPos, const String& tip,
 
         toFront (false);
         manuallyShownTip = shownManually == ShownManually::yes ? tip : String();
-        dismissalMouseEventOccured = false;
+        dismissalMouseEventOccurred = false;
     }
 }
 
@@ -168,7 +168,7 @@ void TooltipWindow::hideTip()
     {
         tipShowing = {};
         manuallyShownTip = {};
-        dismissalMouseEventOccured = false;
+        dismissalMouseEventOccurred = false;
 
         removeFromDesktop();
         setVisible (false);
@@ -201,7 +201,7 @@ void TooltipWindow::timerCallback()
 
     if (manuallyShownTip.isNotEmpty())
     {
-        if (dismissalMouseEventOccured || newComp == nullptr)
+        if (dismissalMouseEventOccurred || newComp == nullptr)
             hideTip();
 
         return;
@@ -218,7 +218,7 @@ void TooltipWindow::timerCallback()
         const auto tipChanged = (newTip != lastTipUnderMouse || newComp != lastComponentUnderMouse);
         const auto now = Time::getApproximateMillisecondCounter();
 
-        if (tipChanged || dismissalMouseEventOccured || mouseMovedQuickly)
+        if (tipChanged || dismissalMouseEventOccurred || mouseMovedQuickly)
             lastCompChangeTime = now;
 
         const auto showTip = [this, &mouseSource, &mousePos, &newTip]
@@ -231,7 +231,7 @@ void TooltipWindow::timerCallback()
         {
             // if a tip is currently visible (or has just disappeared), update to a new one
             // immediately if needed..
-            if (newComp == nullptr || dismissalMouseEventOccured || newTip.isEmpty())
+            if (newComp == nullptr || dismissalMouseEventOccurred || newTip.isEmpty())
                 hideTip();
             else if (tipChanged)
                 showTip();
