@@ -28,9 +28,9 @@ namespace VideoRenderers
     //==============================================================================
     struct Base
     {
-        virtual ~Base() {}
+        virtual ~Base() = default;
 
-        virtual HRESULT create (ComSmartPtr<IGraphBuilder>&, ComSmartPtr<IBaseFilter>&, HWND) = 0;
+        virtual HRESULT create (ComSmartPtr<ComTypes::IGraphBuilder>&, ComSmartPtr<ComTypes::IBaseFilter>&, HWND) = 0;
         virtual void setVideoWindow (HWND) = 0;
         virtual void setVideoPosition (HWND) = 0;
         virtual void repaintVideo (HWND, HDC) = 0;
@@ -43,19 +43,19 @@ namespace VideoRenderers
     {
         VMR7() {}
 
-        HRESULT create (ComSmartPtr<IGraphBuilder>& graphBuilder,
-                        ComSmartPtr<IBaseFilter>& baseFilter, HWND hwnd) override
+        HRESULT create (ComSmartPtr<ComTypes::IGraphBuilder>& graphBuilder,
+                        ComSmartPtr<ComTypes::IBaseFilter>& baseFilter, HWND hwnd) override
         {
-            ComSmartPtr<IVMRFilterConfig> filterConfig;
+            ComSmartPtr<ComTypes::IVMRFilterConfig> filterConfig;
 
-            HRESULT hr = baseFilter.CoCreateInstance (CLSID_VideoMixingRenderer);
+            HRESULT hr = baseFilter.CoCreateInstance (ComTypes::CLSID_VideoMixingRenderer);
 
             if (SUCCEEDED (hr))   hr = graphBuilder->AddFilter (baseFilter, L"VMR-7");
             if (SUCCEEDED (hr))   hr = baseFilter.QueryInterface (filterConfig);
-            if (SUCCEEDED (hr))   hr = filterConfig->SetRenderingMode (VMRMode_Windowless);
+            if (SUCCEEDED (hr))   hr = filterConfig->SetRenderingMode (ComTypes::VMRMode_Windowless);
             if (SUCCEEDED (hr))   hr = baseFilter.QueryInterface (windowlessControl);
             if (SUCCEEDED (hr))   hr = windowlessControl->SetVideoClippingWindow (hwnd);
-            if (SUCCEEDED (hr))   hr = windowlessControl->SetAspectRatioMode (VMR_ARMODE_LETTER_BOX);
+            if (SUCCEEDED (hr))   hr = windowlessControl->SetAspectRatioMode (ComTypes::VMR_ARMODE_LETTER_BOX);
 
             return hr;
         }
@@ -92,7 +92,7 @@ namespace VideoRenderers
             return windowlessControl->GetNativeVideoSize (&videoWidth, &videoHeight, nullptr, nullptr);
         }
 
-        ComSmartPtr<IVMRWindowlessControl> windowlessControl;
+        ComSmartPtr<ComTypes::IVMRWindowlessControl> windowlessControl;
 
         JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (VMR7)
     };
@@ -101,21 +101,23 @@ namespace VideoRenderers
     //==============================================================================
     struct EVR  : public Base
     {
-        EVR() {}
+        EVR() = default;
 
-        HRESULT create (ComSmartPtr<IGraphBuilder>& graphBuilder,
-                        ComSmartPtr<IBaseFilter>& baseFilter, HWND hwnd) override
+        HRESULT create (ComSmartPtr<ComTypes::IGraphBuilder>& graphBuilder,
+                        ComSmartPtr<ComTypes::IBaseFilter>& baseFilter, HWND hwnd) override
         {
-            ComSmartPtr<IMFGetService> getService;
+            ComSmartPtr<ComTypes::IMFGetService> getService;
 
-            HRESULT hr = baseFilter.CoCreateInstance (CLSID_EnhancedVideoRenderer);
+            HRESULT hr = baseFilter.CoCreateInstance (ComTypes::CLSID_EnhancedVideoRenderer);
 
             if (SUCCEEDED (hr))   hr = graphBuilder->AddFilter (baseFilter, L"EVR");
             if (SUCCEEDED (hr))   hr = baseFilter.QueryInterface (getService);
-            if (SUCCEEDED (hr))   hr = getService->GetService (MR_VIDEO_RENDER_SERVICE, IID_IMFVideoDisplayControl,
+            JUCE_BEGIN_IGNORE_WARNINGS_GCC_LIKE ("-Wlanguage-extension-token")
+            if (SUCCEEDED (hr))   hr = getService->GetService (ComTypes::MR_VIDEO_RENDER_SERVICE, __uuidof (ComTypes::IMFVideoDisplayControl),
                                                                (void**) videoDisplayControl.resetAndGetPointerAddress());
+            JUCE_END_IGNORE_WARNINGS_GCC_LIKE
             if (SUCCEEDED (hr))   hr = videoDisplayControl->SetVideoWindow (hwnd);
-            if (SUCCEEDED (hr))   hr = videoDisplayControl->SetAspectRatioMode (MFVideoARMode_PreservePicture);
+            if (SUCCEEDED (hr))   hr = videoDisplayControl->SetAspectRatioMode (ComTypes::MFVideoARMode_PreservePicture);
 
             return hr;
         }
@@ -127,7 +129,7 @@ namespace VideoRenderers
 
         void setVideoPosition (HWND hwnd) override
         {
-            const MFVideoNormalizedRect src = { 0.0f, 0.0f, 1.0f, 1.0f };
+            const ComTypes::MFVideoNormalizedRect src { 0.0f, 0.0f, 1.0f, 1.0f };
 
             RECT dest;
             GetClientRect (hwnd, &dest);
@@ -151,7 +153,7 @@ namespace VideoRenderers
             return hr;
         }
 
-        ComSmartPtr<IMFVideoDisplayControl> videoDisplayControl;
+        ComSmartPtr<ComTypes::IMFVideoDisplayControl> videoDisplayControl;
 
         JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (EVR)
     };
@@ -358,6 +360,7 @@ private:
         {
         }
 
+        using ComponentMovementWatcher::componentMovedOrResized;
         void componentMovedOrResized (bool, bool) override
         {
             if (owner.videoLoaded)
@@ -373,6 +376,7 @@ private:
                 owner.recreateNativeWindowAsync();
         }
 
+        using ComponentMovementWatcher::componentVisibilityChanged;
         void componentVisibilityChanged() override
         {
             if (owner.videoLoaded)
@@ -442,8 +446,8 @@ private:
 
             createNativeWindow();
 
-            mediaEvent->CancelDefaultHandling (EC_STATE_CHANGE);
-            mediaEvent->SetNotifyWindow ((OAHWND) hwnd, graphEventID, 0);
+            mediaEvent->CancelDefaultHandling (ComTypes::EC_STATE_CHANGE);
+            mediaEvent->SetNotifyWindow ((ComTypes::OAHWND) hwnd, graphEventID, 0);
 
             if (videoRenderer != nullptr)
                 videoRenderer->setVideoWindow (hwnd);
@@ -493,7 +497,7 @@ private:
             if (! createNativeWindow())
                 return Result::fail ("Can't create window");
 
-            HRESULT hr = graphBuilder.CoCreateInstance (CLSID_FilterGraph);
+            HRESULT hr = graphBuilder.CoCreateInstance (ComTypes::CLSID_FilterGraph);
 
             // basic playback interfaces
             if (SUCCEEDED (hr))   hr = graphBuilder.QueryInterface (mediaControl);
@@ -555,8 +559,8 @@ private:
             // set window to receive events
             if (SUCCEEDED (hr))
             {
-                mediaEvent->CancelDefaultHandling (EC_STATE_CHANGE);
-                hr = mediaEvent->SetNotifyWindow ((OAHWND) hwnd, graphEventID, 0);
+                mediaEvent->CancelDefaultHandling (ComTypes::EC_STATE_CHANGE);
+                hr = mediaEvent->SetNotifyWindow ((ComTypes::OAHWND) hwnd, graphEventID, 0);
             }
 
             if (SUCCEEDED (hr))
@@ -577,18 +581,18 @@ private:
         {
             switch (hr)
             {
-                case VFW_E_INVALID_FILE_FORMAT:         return Result::fail ("Invalid file format");
-                case VFW_E_NOT_FOUND:                   return Result::fail ("File not found");
-                case VFW_E_UNKNOWN_FILE_TYPE:           return Result::fail ("Unknown file type");
-                case VFW_E_UNSUPPORTED_STREAM:          return Result::fail ("Unsupported stream");
-                case VFW_E_CANNOT_CONNECT:              return Result::fail ("Cannot connect");
-                case VFW_E_CANNOT_LOAD_SOURCE_FILTER:   return Result::fail ("Cannot load source filter");
+                case ComTypes::VFW_E_INVALID_FILE_FORMAT:         return Result::fail ("Invalid file format");
+                case ComTypes::VFW_E_NOT_FOUND:                   return Result::fail ("File not found");
+                case ComTypes::VFW_E_UNKNOWN_FILE_TYPE:           return Result::fail ("Unknown file type");
+                case ComTypes::VFW_E_UNSUPPORTED_STREAM:          return Result::fail ("Unsupported stream");
+                case ComTypes::VFW_E_CANNOT_CONNECT:              return Result::fail ("Cannot connect");
+                case ComTypes::VFW_E_CANNOT_LOAD_SOURCE_FILTER:   return Result::fail ("Cannot load source filter");
             }
 
             TCHAR messageBuffer[512] = { 0 };
 
             FormatMessage (FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
-                           nullptr, hr, MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT),
+                           nullptr, (DWORD) hr, MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT),
                            messageBuffer, (DWORD) numElementsInArray (messageBuffer) - 1, nullptr);
 
             return Result::fail (String (messageBuffer));
@@ -633,28 +637,28 @@ private:
 
                 switch (ec)
                 {
-                    case EC_REPAINT:
+                    case ComTypes::EC_REPAINT:
                         component.repaint();
                         break;
 
-                    case EC_COMPLETE:
+                    case ComTypes::EC_COMPLETE:
                         component.stop();
                         component.setPosition (0.0);
                         break;
 
-                    case EC_ERRORABORT:
-                    case EC_ERRORABORTEX:
+                    case ComTypes::EC_ERRORABORT:
+                    case ComTypes::EC_ERRORABORTEX:
                         component.errorOccurred (getErrorMessageFromResult ((HRESULT) p1).getErrorMessage());
                         // intentional fallthrough
-                    case EC_USERABORT:
+                    case ComTypes::EC_USERABORT:
                         component.close();
                         break;
 
-                    case EC_STATE_CHANGE:
+                    case ComTypes::EC_STATE_CHANGE:
                         switch (p1)
                         {
-                            case State_Paused:  component.playbackStopped(); break;
-                            case State_Running: component.playbackStarted(); break;
+                            case ComTypes::State_Paused:  component.playbackStopped(); break;
+                            case ComTypes::State_Running: component.playbackStarted(); break;
                             default: break;
                         }
 
@@ -697,7 +701,7 @@ private:
         //==============================================================================
         double getDuration() const
         {
-            REFTIME duration;
+            ComTypes::REFTIME duration;
             mediaPosition->get_Duration (&duration);
             return duration;
         }
@@ -711,7 +715,7 @@ private:
 
         double getPosition() const
         {
-            REFTIME seconds;
+            ComTypes::REFTIME seconds;
             mediaPosition->get_CurrentPosition (&seconds);
             return seconds;
         }
@@ -747,12 +751,12 @@ private:
         HWND hwnd = {};
         HDC hdc = {};
 
-        ComSmartPtr<IGraphBuilder> graphBuilder;
-        ComSmartPtr<IMediaControl> mediaControl;
-        ComSmartPtr<IMediaPosition> mediaPosition;
-        ComSmartPtr<IMediaEventEx> mediaEvent;
-        ComSmartPtr<IBasicAudio> basicAudio;
-        ComSmartPtr<IBaseFilter> baseFilter;
+        ComSmartPtr<ComTypes::IGraphBuilder> graphBuilder;
+        ComSmartPtr<ComTypes::IMediaControl> mediaControl;
+        ComSmartPtr<ComTypes::IMediaPosition> mediaPosition;
+        ComSmartPtr<ComTypes::IMediaEventEx> mediaEvent;
+        ComSmartPtr<ComTypes::IBasicAudio> basicAudio;
+        ComSmartPtr<ComTypes::IBaseFilter> baseFilter;
 
         std::unique_ptr<VideoRenderers::Base> videoRenderer;
 
@@ -800,31 +804,31 @@ private:
 
         bool isRendererConnected()
         {
-            ComSmartPtr<IEnumPins> enumPins;
+            ComSmartPtr<ComTypes::IEnumPins> enumPins;
 
             HRESULT hr = baseFilter->EnumPins (enumPins.resetAndGetPointerAddress());
 
             if (SUCCEEDED (hr))
                 hr = enumPins->Reset();
 
-            ComSmartPtr<IPin> pin;
+            ComSmartPtr<ComTypes::IPin> pin;
 
             while (SUCCEEDED (hr)
                     && enumPins->Next (1, pin.resetAndGetPointerAddress(), nullptr) == S_OK)
             {
-                ComSmartPtr<IPin> otherPin;
+                ComSmartPtr<ComTypes::IPin> otherPin;
 
                 hr = pin->ConnectedTo (otherPin.resetAndGetPointerAddress());
 
                 if (SUCCEEDED (hr))
                 {
-                    PIN_DIRECTION direction;
+                    ComTypes::PIN_DIRECTION direction;
                     hr = pin->QueryDirection (&direction);
 
-                    if (SUCCEEDED (hr) && direction == PINDIR_INPUT)
+                    if (SUCCEEDED (hr) && direction == ComTypes::PINDIR_INPUT)
                         return true;
                 }
-                else if (hr == VFW_E_NOT_CONNECTED)
+                else if (hr == ComTypes::VFW_E_NOT_CONNECTED)
                 {
                     hr = S_OK;
                 }
